import os

os.system("export PY_IGNORE_IMPORTMISMATCH=1")
from EUKulele.EUKulele_config import *
from EUKulele.EUKulele_main import *
#rc = os.system("which conda")
#if rc == 0:
#    print("Conda found on system; conda environment will be automatically created and activated.")
#    os.system("conda env create -f EUKulele-env.yaml --force")
#    os.system("conda activate EUKulele")
#else:
#    print("Conda environment not successfully created. Before running the software, you should make sure non-Python dependencies are installed. BLAST and/or DIAMOND (depending on alignment choice), and TransDecoder must be installed.")



#!/usr/bin/env python
import ujson
import pandas as pd
import numpy as np
import yaml
import chardet
import argparse
import multiprocessing
import os
from joblib import Parallel, delayed

import EUKulele

def tax_placement(pident, tax_cutoffs):
    if pident >= tax_cutoffs['species']:
        out = 'species'; level = 7;
    elif pident >= tax_cutoffs['genus']:
        out = 'genus'; level = 6;
    elif pident >= tax_cutoffs['family']:
        out = 'family'; level = 5;
    elif pident >= tax_cutoffs['order']:
        out = 'order'; level = 4;
    elif pident < tax_cutoffs['order']:
        out = 'class'; level = 3;
    return out, level

def read_in_taxonomy(infile):
    with open(infile, 'rb') as f:
        result = chardet.detect(f.read())
    tax_out = pd.read_csv(infile, sep='\t',encoding=result['encoding'])
    classes = ['supergroup','division','class','order','family','genus','species']
    for c in tax_out.columns:
        if c.lower() in classes:
            if (np.issubdtype(tax_out[str(c)].dtypes, np.number)) | (np.issubdtype(tax_out[str(c)].dtypes, np.float_)):
                tax_out = tax_out.loc[:,~(tax_out.columns == c)]
    tax_out.columns = tax_out.columns.str.lower()
    tax_out = tax_out.set_index('source_id')
    return tax_out

def read_in_tax_cutoffs(yamlfile):
    with open(yamlfile, 'r') as stream:
        co_out = yaml.safe_load(stream)
    return co_out

def read_in_protein_map(protjson):
    with open(protjson, 'rb') as f:
        pout = ujson.load(f)
    return pout

def gen_dict(tax_table):
    classes = ['supergroup','division','class','order','family','genus','species']
    tax_table["Classification"] = ""
    tax_table = tax_table.loc[:,~tax_table.columns.duplicated()]
    for c in classes:
        if all([str(curr).lower() != "nan" for curr in list(tax_table[c])]):
            if str(tax_table["Classification"][0]) != "":
                tax_table["Classification"] = tax_table["Classification"] + ";" + tax_table[c]
            else:
                tax_table["Classification"] = tax_table["Classification"] + tax_table[c]
    return(dict(zip(tax_table.index, tax_table["Classification"])))

def gen_reads_dict(names_to_reads):
    names_to_reads = pd.read_csv(names_to_reads,header=0,sep="\t")
    return(dict(zip(names_to_reads["TranscriptNames"],names_to_reads["NumReads"])))

def lca(full_classifications):
    classes = ['supergroup','division','class','order','family','genus','species']
    full_classifications_split = [[str(subtax).strip() for subtax in curr.split(";")] for curr in full_classifications]
    length_classes = [len(curr) for curr in full_classifications_split]
    if len(set(length_classes)) != 1:
        print("Error: not all classifications at at the same taxonomic level.")
        sys.exit(1)
    for l in reversed(range(length_classes[0])):
        set_classifications = [curr[l] for curr in full_classifications_split]
        if len(set(set_classifications)) == 1:
            return classes[l], set_classifications[0], "; ".join(full_classifications_split[0][0:(l+1)])
    return "","","" # if there are no common ancestors

def match_maker(dd, consensus_cutoff, tax_dict, use_counts, tax_cutoffs):
    ambiguous = 0 # we assume unambiguous
    md = dd.pident.max()
    transcript_name = set(list(dd["qseqid"]))
    if len(transcript_name) > 1:
        print("More than 1 transcript name included in the group.")
    transcript_name = list(transcript_name)[0]
    ds = list(set(dd[dd.pident==md]['ssqid_TAXID']))
    counts = list(set(dd[dd.pident==md]['counts']))
    if (len(counts) >= 1):
        chosen_count = counts[0]
    else:
        chosen_count = 0
    assignment, level = tax_placement(md, tax_cutoffs) # most specific taxonomic level assigned
    if len(ds)==1:
        if ds[0] not in tax_dict:
            return(pd.DataFrame(columns=['transcript_name','classification_level', 'full_classification', 
                                         'classification', 'max_pid', 'counts', 'ambiguous']))
        full_classification = str(tax_dict[ds[0]]).split(";")[0:level]
        best_classification = full_classification[len(full_classification) - 1] # the most specific taxonomic level we can classify by
        full_classification = '; '.join(full_classification) # the actual assignments based on that
    else:
        classification_0 = []
        full_classification_0 = []
        for d in ds:
            if d not in tax_dict:
                return(pd.DataFrame(columns=['transcript_name','classification_level', 'full_classification', 
                                             'classification', 'max_pid', 'counts', 'ambiguous']))
            d_full_class = str(tax_dict[d]).split(";")[0:(level + 1)]
            classification_0.append(d_full_class[len(d_full_class) - 1]) # the most specific taxonomic level we can classify by
            full_classification_0.append('; '.join(d_full_class)) # the actual assignments based on that
        entries = list(set(full_classification_0))
        if len(entries) == 1:
            best_classification = classification_0[0]
            full_classification = full_classification_0[0]
        else:
            ambiguous = 1
            best_frac = 0
            best_one_class = 0
            best_full_class = 0
            for e in entries:
                curr_frac = len(np.where(full_classification_0 == e)) / len(full_classification_0)
                if (isinstance(curr_frac, float)) & (curr_frac > best_frac):
                    best_frac = curr_frac
                    best_full_class = e
                    best_one_class = str(e.split(";")[len(e.split(";")) - 1]).strip()
            if best_frac >= consensus_cutoff:
                best_classification = best_one_class
                full_classification = best_full_class
            else:
                assignment, best_classification, full_classification = lca(full_classification_0)

    if use_counts == 1:
        return pd.DataFrame([[transcript_name, assignment, full_classification, best_classification, md, chosen_count, ambiguous]],\
                       columns=['transcript_name','classification_level', 'full_classification', 
                                'classification', 'max_pid', 'counts', 'ambiguous'])
    else:
        return pd.DataFrame([[transcript_name, assignment, full_classification, best_classification, md, ambiguous]],\
                       columns=['transcript_name', 'classification_level', 'full_classification', 
                                'classification', 'max_pid', 'ambiguous'])

def apply_parallel(grouped_data, match_maker, consensus_cutoff, tax_dict, use_counts, tax_cutoffs):
    resultdf = Parallel(n_jobs=multiprocessing.cpu_count(), prefer="threads")(delayed(match_maker)(group, consensus_cutoff, 
                                                                                                   tax_dict, use_counts, 
                                                                                                   tax_cutoffs) for name, group \
                                                                              in grouped_data)
    return pd.concat(resultdf)

def classify_taxonomy_parallel(df, tax_dict, namestoreads, pdict, consensus_cutoff, tax_cutoffs):
    chunksize = 10 ** 6
    counter = 0
    for chunk in pd.read_csv(str(df), sep = '\t', header = None, chunksize=chunksize):
        chunk.columns = ['qseqid', 'sseqid', 'pident', 'length', 'mismatch', 'gapopen', 'qstart', 
                         'qend', 'sstart', 'send', 'evalue', 'bitscore']
        chunk['ssqid_TAXID']=chunk.sseqid.map(pdict)
        if namestoreads != 0:
            chunk['counts']=[namestoreads[curr.split(".")[0]] if curr.split(".")[0] in namestoreads else 0 for curr in chunk.qseqid]
            use_counts = 1
        else:
            chunk['counts'] = [0] * len(chunk.qseqid) # if no reads dict, each count is just assumed to be 0 and isn't recorded later
            use_counts = 0
            
        print(chunk, flush=True)
        if counter == 0:
            outdf = apply_parallel(chunk.groupby('qseqid'), match_maker, 
                                   consensus_cutoff, tax_dict, use_counts, tax_cutoffs)
        else:
            outdf = pd.concat([outdf, apply_parallel(chunk.groupby('qseqid'), 
                                                     match_maker, consensus_cutoff, tax_dict, 
                                                     use_counts, tax_cutoffs)], axis = 0)
        counter = counter + 1
    return outdf

def place_taxonomy(tax_file,cutoff_file,consensus_cutoff,prot_map_file,
                   use_counts,names_to_reads,diamond_file,outfile,rerun):
    if (os.path.isfile(outfile)) & (not rerun):
        print("Taxonomic placement already complete; will not re-run step.")
        return pd.read_csv(outfile, sep = "\t")
    
    tax_table = read_in_taxonomy(tax_file)
    tax_cutoffs = read_in_tax_cutoffs(os.path.join(os.path.dirname(os.path.realpath(__file__)), "static", cutoff_file))
    pdict = read_in_protein_map(prot_map_file)
    tax_dict = gen_dict(tax_table)
    consensus_cutoff = float(consensus_cutoff)
    if (int(use_counts) == 1):
        reads_dict = gen_reads_dict(names_to_reads)
        classification_df = classify_taxonomy_parallel(diamond_file, tax_dict, reads_dict, 
                                                       pdict, consensus_cutoff, tax_cutoffs)
    else:
        classification_df = classify_taxonomy_parallel(diamond_file, tax_dict, 0, pdict, 
                                                       consensus_cutoff, tax_cutoffs)
    classification_df.to_csv(outfile, sep='\t')
    return outfile



#!/usr/bin/env python

# Software for taxonomic identification of eukaryotes.

import pandas as pd
import numpy as np
import os
import sys
import yaml
import chardet
import argparse
import multiprocessing
import subprocess
import shutil
import glob
from joblib import Parallel, delayed
import pkgutil

import EUKulele

from EUKulele.download_database import downloadDatabase
from EUKulele.manage_steps import manageEukulele
from EUKulele.busco_runner import readBuscoFile
from EUKulele.busco_runner import configRunBusco
from EUKulele.busco_runner import manageBuscoQuery

import scripts as HelperScripts
from scripts.names_to_reads import namesToReads

__author__ = "Harriet Alexander, Arianna Krinos"
__copyright__ = "EUKulele"
__license__ = "MIT"
__email__ = "akrinos@mit.edu"

def main(args_in):
    parser = argparse.ArgumentParser(
        description='Thanks for using EUKulele! EUKulele is a standalone taxonomic annotation software.\n'
                    'EUKulele is designed primarily for marine microbial eukaryotes. Check the README '
                    'for further information.',
        usage='eukulele [subroutine] --mets_or_mags [dataset_type] --sample_dir [sample_directory] ' + 
              '--reference_dir [reference_database_location] [all other options]')
    
    parser.add_argument('subroutine', metavar="subroutine", nargs='?', type=str, default="all", 
                        choices = ["","all","setup","alignment","busco","coregenes"], 
                        help='Choice of subroutine to run.')
    parser.add_argument('-m', '--mets_or_mags', dest = "mets_or_mags", required = True) 
    parser.add_argument('--n_ext', '--nucleotide_extension', dest = "nucleotide_extension", default = ".fasta") 
    parser.add_argument('--p_ext', '--protein_extension', dest = "protein_extension", default = ".faa") 
    parser.add_argument('-f', '--force_rerun', action='store_true', default=False)
    parser.add_argument('--scratch', default = '../scratch', 
                        help = "The scratch location to store intermediate files.")
    parser.add_argument('--config_file', default = '')

    ## SALMON OPTIONS ##
    parser.add_argument('--use_salmon_counts', action='store_true', default=False)
    parser.add_argument('--salmon_dir', 
                        help = "Salmon directory is required if use_salmon_counts is true.")
    parser.add_argument('--names_to_reads',default=0, help = "A file to be created or used if it exists " +
                        "that relates transcript names to salmon counts from the salmon directory.")

    ## WHERE FILES ARE LOCATED ##
    parser.add_argument('-d', '--database', default="mmetsp", 
                        help = "The name of the database to be used to assess the reads.")
    parser.add_argument('-o', '--out_dir', dest = "out_dir", default = "output", 
                        help = "Folder where the output will be written.")
    parser.add_argument('-s', '--sample_dir', required = True, dest = "sample_dir",
                        help = "Folder where the input data is located (the protein or peptide files to be assessed).")
    
    ## ONLY SPECIFY THESE ARGUMENTS IF YOU HAVE ALREADY PROVIDED AND FORMATTED YOUR OWN DATABASE ##
    parser.add_argument('--reference_dir', default="", 
                        help = "Folder containing the reference files for the chosen database.")
    parser.add_argument('--ref_fasta', default = "reference.pep.fa", 
                        help = "Either a file in the reference directory where the fasta file for the database " + 
                               "is located, or a directory containing multiple fasta files that " + 
                               "constitute the database.")
  
    parser.add_argument('--tax_table', default = "tax-table.txt")
    parser.add_argument('--protein_map', default = "prot-map.json")
    
    ## ALIGNMENT OPTIONS ##
    parser.add_argument('--alignment_choice', default = "diamond", choices = ["diamond", "blast"])

    ## OPTIONS FOR CHECKING BUSCO COMPLETENESS FOR TAXONOMY ##
    parser.add_argument('--busco_file', default = "", type = str, 
                        help = "If specified, the following two arguments ('--organisms' and " + 
                        "'--taxonomy_organisms' are overwritten by the two columns of this tab-separated file.")
    parser.add_argument('--individual_or_summary', default="summary", choices=["summary","individual"], 
                        help = "These arguments are used if 'individual' is specified.") 
    parser.add_argument('-i', '--individual', dest = "individual_tag", action='store_true', default=False)
    parser.add_argument('--organisms', default = "", nargs = "+", 
                        help = "List of organisms to check BUSCO completeness on.")
    parser.add_argument('--taxonomy_organisms', default = "", nargs = "+", 
                        help = "Taxonomic level of organisms specified in organisms tag.")
    parser.add_argument('--test', action='store_true', default=False)

    ## OTHER USER CHOICES ## 
    #cutoff_file = pkgutil.get_data(__name__, "tax-cutoffs.yaml")
    cutoff_file = "tax-cutoffs.yaml"
    parser.add_argument('--cutoff_file', default = cutoff_file)
    parser.add_argument('--filter_metric', default = "evalue", choices = ["pid", "evalue", "bitscore"])
    parser.add_argument('--consensus_cutoff', default = 0.75, type = float)
    parser.add_argument('--transdecoder_orfsize', default = 100, type = int)

    parser.add_argument('--CPUs', default=multiprocessing.cpu_count())
    parser.add_argument('--busco_threshold', default=50)
    parser.add_argument('--create_fasta', action='store_true', default=False, 
                       help = "Whether to create FASTA files containing ID'd transcripts during BUSCO analysis.")
    parser.add_argument('--run_transdecoder', action='store_true', default=False,
                       help = "Whether TransDecoder should be run on metatranscriptomic samples. Otherwise, " +
                       "BLASTp is run if protein translated samples are provided, otherwise BLASTx is run " + 
                       "on nucleotide samples.")
               
    args = parser.parse_args(list(filter(None, args_in.split(" "))))
    
    ## VARIABLES ##
    TEST = args.test
    CONSENSUS_CUTOFF = args.consensus_cutoff
    REFERENCE_DIR = args.reference_dir
    OUTPUTDIR = args.out_dir
    SAMPLE_DIR = args.sample_dir
    REF_FASTA = args.ref_fasta

    ALIGNMENT_CHOICE = args.alignment_choice
    OUTPUT_EXTENSION = "txt"
    DBEXTENSION = ""
    TRANSDECODERORFSIZE=args.transdecoder_orfsize
    if ALIGNMENT_CHOICE == "diamond":
        OUTPUT_EXTENSION = "out"
        DBEXTENSION = ".dmnd"
    NT_EXT = args.nucleotide_extension.strip('.')
    PEP_EXT = args.protein_extension.strip('.')
    mets_or_mags = args.mets_or_mags.lower()
    individual_or_summary = args.individual_or_summary
    if args.individual_tag:
        individual_or_summary = "individual"
    
    if (mets_or_mags != "mets") & (mets_or_mags != "mags"):
        print("Only METs or MAGs are supported as input data types. Please update the 'mets_or_mags' flag accordingly.")
        sys.exit(1)

    USE_SALMON_COUNTS = args.use_salmon_counts
    SALMON_DIR = args.salmon_dir
    NAMES_TO_READS = os.path.join(REFERENCE_DIR, str(args.names_to_reads))
    CPUS = args.CPUs                         

    ORGANISMS = args.organisms 
    ORGANISMS_TAXONOMY = args.taxonomy_organisms
    BUSCO_FILE = args.busco_file
    RERUN_RULES = args.force_rerun
    RUN_TRANSDECODER = args.run_transdecoder
    
    ORGANISMS, ORGANISMS_TAXONOMY = readBuscoFile(individual_or_summary, BUSCO_FILE, 
                                                  ORGANISMS, ORGANISMS_TAXONOMY)

    SETUP = False
    ALIGNMENT = False
    BUSCO = False
    COREGENES = False
    if (args.subroutine == "all") | (args.subroutine == "setup"):
        SETUP = True
    if (args.subroutine == "all") | (args.subroutine == "alignment"):
        ALIGNMENT = True
    if (args.subroutine == "all") | (args.subroutine == "busco"):
        BUSCO = True
    if (args.subroutine == "all") | (args.subroutine == "coregenes"):
        COREGENES = True
        
    if TEST:
        SETUP = False
        ALIGNMENT = False
        BUSCO = False
        COREGENES = False
    else: 
        ## SETUP STEPS / DOWNLOAD DEPENDENCIES ##
        manageEukulele(piece = "setup_eukulele", output_dir = OUTPUTDIR)
        samples = manageEukulele(piece = "get_samples", mets_or_mags = mets_or_mags,
                                 sample_dir = SAMPLE_DIR, nt_ext = NT_EXT, pep_ext = PEP_EXT)

        TAX_TAB = os.path.join(REFERENCE_DIR, args.tax_table)
        PROT_TAB = os.path.join(REFERENCE_DIR, args.protein_map)

        ## Download the reference database if specified.
        if (not os.path.isfile(os.path.join(REFERENCE_DIR, REF_FASTA))) | \
           (not os.path.isfile(TAX_TAB)) | \
           (not os.path.isfile(PROT_TAB)):
            REFERENCE_DIR = args.database.lower()
            print("Specified reference directory, reference FASTA, and protein map/taxonomy table not found. " +
                  "Using database: " + REFERENCE_DIR + ".")
            os.system("mkdir -p " + REFERENCE_DIR)
            TAX_TAB = os.path.join(REFERENCE_DIR, "tax-table.txt")
            PROT_TAB = os.path.join(REFERENCE_DIR, "prot-map.json")

        if (not os.path.isfile(os.path.join(REFERENCE_DIR, REF_FASTA))) | \
           (not os.path.isfile(TAX_TAB)) | \
           (not os.path.isfile(PROT_TAB)):
            REF_FASTA, TAX_TAB, PROT_TAB = downloadDatabase(args.database.lower())
        else:
            print("Found database folder for " + REFERENCE_DIR + " in current directory; will not re-download.")

    if SETUP:
        manageEukulele(piece = "setup_databases", ref_fasta = REF_FASTA, rerun_rules = RERUN_RULES, 
                       alignment_choice = ALIGNMENT_CHOICE, database_dir = REFERENCE_DIR)

    if ALIGNMENT:
        ## First, we need to perform TransDecoder if needed
        manageEukulele(piece = "transdecode", mets_or_mags = mets_or_mags, samples = samples, output_dir = OUTPUTDIR, 
                       rerun_rules = RERUN_RULES, sample_dir = SAMPLE_DIR, transdecoder_orf_size = TRANSDECODERORFSIZE, 
                       nt_ext = NT_EXT, pep_ext = PEP_EXT, run_transdecoder = RUN_TRANSDECODER)
        
        ## Next to align against our database of choice ##
        alignment_res = manageEukulele(piece = "align_to_db", alignment_choice = ALIGNMENT_CHOICE, samples = samples, 
                                        filter_metric = args.filter_metric, output_dir = OUTPUTDIR, 
                                        ref_fasta = REF_FASTA, mets_or_mags = mets_or_mags, database_dir = REFERENCE_DIR,
                                        sample_dir = SAMPLE_DIR, rerun_rules = RERUN_RULES, 
                                        nt_ext = NT_EXT, pep_ext = PEP_EXT)

        ## Next to do salmon counts estimation. ##
        if (USE_SALMON_COUNTS == 1):
            try:
                NAMES_TO_READS = namesToReads(args.config_file)
            except:
                print("The salmon directory provided could not be converted to a salmon file. Check " +
                      "above error messages. EUKulele will continue running without counts.")
                USE_SALMON_COUNTS = 0

        manageEukulele(piece = "estimate_taxonomy", output_dir = OUTPUTDIR, mets_or_mags = mets_or_mags, 
                       tax_tab = TAX_TAB, cutoff_file = args.cutoff_file, 
                       consensus_cutoff = CONSENSUS_CUTOFF, prot_tab = PROT_TAB, use_salmon_counts = USE_SALMON_COUNTS, 
                       names_to_reads = NAMES_TO_READS, alignment_res = alignment_res, 
                       rerun_rules = RERUN_RULES, samples = samples, sample_dir = SAMPLE_DIR, pep_ext = PEP_EXT,
                       nt_ext = NT_EXT)

        ## Now to visualize the taxonomy ##
        manageEukulele(piece = "visualize_taxonomy", output_dir = OUTPUTDIR, mets_or_mags = mets_or_mags, 
                       sample_dir = SAMPLE_DIR, pep_ext = PEP_EXT, nt_ext = NT_EXT, 
                       use_salmon_counts = USE_SALMON_COUNTS, rerun_rules = RERUN_RULES)

        ## Next to assign taxonomy ##
        manageEukulele(piece = "assign_taxonomy", samples = samples, mets_or_mags = mets_or_mags, 
                       sample_dir = SAMPLE_DIR, pep_ext = PEP_EXT,
                       output_dir = OUTPUTDIR)

    if BUSCO:
        configRunBusco(output_dir = OUTPUTDIR, mets_or_mags = mets_or_mags, pep_ext = PEP_EXT, 
                       nt_ext = NT_EXT, sample_dir = SAMPLE_DIR, samples = samples)

        manageBuscoQuery(output_dir = OUTPUTDIR, individual_or_summary = individual_or_summary, 
                         samples = samples, mets_or_mags = mets_or_mags, pep_ext = PEP_EXT, 
                         nt_ext = NT_EXT, sample_dir = SAMPLE_DIR, organisms = ORGANISMS, 
                         organisms_taxonomy = ORGANISMS_TAXONOMY, tax_tab = TAX_TAB, 
                         busco_threshold = args.busco_threshold)
        
    if COREGENES:
        print("Investigating core genes...")
        ## Next to align against our database of choice ##
        alignment_res = manageEukulele(piece = "core_align_to_db", alignment_choice = ALIGNMENT_CHOICE, samples = samples, 
                                        filter_metric = args.filter_metric, output_dir = OUTPUTDIR, 
                                        ref_fasta = REF_FASTA, mets_or_mags = mets_or_mags, database_dir = REFERENCE_DIR,
                                        sample_dir = SAMPLE_DIR, rerun_rules = RERUN_RULES, 
                                        nt_ext = NT_EXT, pep_ext = PEP_EXT)
        
        manageEukulele(piece = "core_estimate_taxonomy", output_dir = OUTPUTDIR, mets_or_mags = mets_or_mags, 
                       tax_tab = TAX_TAB, cutoff_file = args.cutoff_file, 
                       consensus_cutoff = CONSENSUS_CUTOFF, prot_tab = PROT_TAB, use_salmon_counts = USE_SALMON_COUNTS, 
                       names_to_reads = NAMES_TO_READS, alignment_res = alignment_res, 
                       rerun_rules = RERUN_RULES, samples = samples, sample_dir = SAMPLE_DIR, pep_ext = PEP_EXT,
                       nt_ext = NT_EXT)

        ## Now to visualize the taxonomy ##
        manageEukulele(piece = "core_visualize_taxonomy", output_dir = OUTPUTDIR, mets_or_mags = mets_or_mags, 
                       sample_dir = SAMPLE_DIR, pep_ext = PEP_EXT, nt_ext = NT_EXT, 
                       use_salmon_counts = USE_SALMON_COUNTS, rerun_rules = RERUN_RULES)

        ## Next to assign taxonomy ##
        manageEukulele(piece = "core_assign_taxonomy", samples = samples, mets_or_mags = mets_or_mags, 
                       sample_dir = SAMPLE_DIR, pep_ext = PEP_EXT,
                       output_dir = OUTPUTDIR)
        
        
    print("EUKulele run complete!", flush = True)
                   
if __name__ == "__main__": 
    main(args_in = " ".join(sys.argv[1:]))


import pandas as pd
import os
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import math
import sys
import yaml
import argparse

def countClassifs(level, level_hierarchy, name_level, df):
    set_list = set()
    
    classifications = list(df.loc[df["classification_level"] == level]["classification"])
    counts = list(df.loc[df["classification_level"] == level]["counts"])
    transcript_names = list(df.loc[df["classification_level"] == level]["transcript_name"])
    match_loc = int(np.where([curr == level for curr in level_hierarchy])[0])
    for curr in range(match_loc + 1,len(level_hierarchy)):
        classification_curr = list(df.loc[df["classification_level"] == level_hierarchy[curr]]["full_classification"])
        transcripts_curr = list(df.loc[df["classification_level"] == level_hierarchy[curr]]["transcript_name"])
        correct_index = list(np.where([len(str(cr).split(";")) >= abs(-1-(curr-match_loc)) for cr in classification_curr])[0])
        
        classification_curr = [classification_curr[cr2] for cr2 in correct_index]
        classifs_curr = [str(cr).split(";")[-1-(curr-match_loc)].strip() for cr in classification_curr]
        transcripts_curr = [transcripts_curr[cr2] for cr2 in correct_index]
        counts_curr = list(df.loc[df["classification_level"] == level_hierarchy[curr]]["counts"])
        counts_curr = [counts_curr[cr2] for cr2 in correct_index]
        
        # add to running list
        classifications.extend(classifs_curr)
        transcript_names.extend(transcripts_curr)
        counts.extend(counts_curr)
        
    classifications = [str(cr).strip().strip(""''"]['") for cr in classifications]
    full_list = classifications
    set_list.update(set(classifications))
    
    transcripts_classes = pd.DataFrame({"classifications": classifications, "transcript_names": transcript_names})
    transcripts_classes = transcripts_classes.groupby("classifications").agg({"transcript_names": lambda x: ';'.join(x)})
    transcripts_classes = transcripts_classes.set_index('classifications')
    transcripts_classes = transcripts_classes.loc[list(set_list)]
    
    full_frame = pd.DataFrame({name_level: classifications, "Counts": counts})
    final_frame = full_frame.groupby(name_level)['Counts'].agg(Sum='sum', Count='count')
    final_frame["GroupedTranscripts"] = list(transcripts_classes.transcript_names)
    return classifications, final_frame
    
def countClassifsNoCounts(level, level_hierarchy, name_level, df):
    set_list = set()
    
    classifications = list(df.loc[df["classification_level"] == level]["classification"])
    transcript_names = list(df.loc[df["classification_level"] == level]["transcript_name"])
    match_loc = int(np.where([curr == level.lower() for curr in level_hierarchy])[0])
    
    for curr in range(match_loc + 1,len(level_hierarchy)):
        classification_curr = list(df.loc[df["classification_level"] == level_hierarchy[curr]]["full_classification"])
        transcripts_curr = list(df.loc[df["classification_level"] == level_hierarchy[curr]]["transcript_name"])
        correct_index = list(np.where([len(str(cr).split(";")) >= abs(-1-(curr-match_loc)) for cr in classification_curr])[0])
        
        classification_curr = [classification_curr[cr2] for cr2 in correct_index]
        transcripts_curr = [transcripts_curr[cr2] for cr2 in correct_index]
        classifs_curr = [str(cr).split(";")[-1-(curr-match_loc)].strip() for cr in classification_curr]
        
        transcript_names.extend(transcripts_curr)
        classifications.extend(classifs_curr)
        
    classifications = [str(cr).strip().strip(""''"]['") for cr in classifications]
    full_list = classifications
    set_list.update(set(classifications))
    
    transcripts_classes = pd.DataFrame({"classifications": classifications, "transcript_names": transcript_names})
    transcripts_classes = transcripts_classes.groupby("classifications").agg({"transcript_names": lambda x: ';'.join(x)})
    transcripts_classes.sort_values(by = "classifications", inplace=True)
    
    final_frame = pd.DataFrame({name_level: sorted(list(set_list)), 
                                "Counts": [full_list.count(curr) for curr in sorted(list(set_list))], 
                                "GroupedTranscripts": list(transcripts_classes.transcript_names)})
    
    return classifications, final_frame

def stripClassifData(df, use_counts):
    level_hierarchy = ['supergroup','division','class','order','family','genus','species']
    
    return_dict_list = dict()
    return_dict_frame = dict()
    for curr_level in level_hierarchy:
        if use_counts == 1:
            curr_list, curr_counts = countClassifs(curr_level, level_hierarchy, curr_level.capitalize(), df)
            return_dict_list[curr_level] = curr_list
            return_dict_frame[curr_level] = curr_counts
        else:
            curr_list, curr_counts = countClassifsNoCounts(curr_level, level_hierarchy, curr_level.capitalize(), df)
            return_dict_list[curr_level] = curr_list
            return_dict_frame[curr_level] = curr_counts
    return return_dict_list, return_dict_frame

def createPlotDataFrame(curr_df_start, cutoff_relative = 0.1, transcript_or_counts="NumTranscripts"):
    ## CREATE AGGREGATED COUNTS BY SAMPLE ##
    curr_df_summed = curr_df_start.groupby("Sample")[transcript_or_counts].agg(AllCts='sum')
    curr_df_summed = curr_df_start.join(curr_df_summed,how="left",on="Sample")

    ## CALCULATE RELATIVE COUNTS ## 
    curr_df_summed["Rel_Counts"] = curr_df_summed[transcript_or_counts] / curr_df_summed["AllCts"]
    curr_df_plot = curr_df_summed[["OfInterest","Sample","Rel_Counts"]]
    pivoted = curr_df_plot.pivot(index='Sample', columns='OfInterest', values='Rel_Counts')
    pivoted = pivoted.reindex(sorted(pivoted.columns), axis=1)

    ## If we're plotting both counts and transcripts, still decide on cutoff via transcripts ## 
    if transcript_or_counts == "Counts":
        curr_df_summed = curr_df_start.groupby("Sample")["NumTranscripts"].agg(AllCts='sum')
        curr_df_summed = curr_df_start.join(curr_df_summed,how="left",on="Sample")
        curr_df_summed["Rel_Counts_Transcripts"] = curr_df_summed["NumTranscripts"] / curr_df_summed["AllCts"]
        curr_df_summed = curr_df_summed[["OfInterest","Sample","Rel_Counts_Transcripts"]]
        pivoted_on_transcripts = curr_df_summed.pivot(index='Sample', columns='OfInterest', values='Rel_Counts_Transcripts')
        pivoted_on_transcripts = pivoted_on_transcripts.reindex(sorted(pivoted_on_transcripts.columns), axis=1)
        pivoted_agg = list(pivoted_on_transcripts.max(axis = 0, skipna = True))
    else:
        pivoted_agg = list(pivoted.max(axis = 0, skipna = True))
    
    ## Leave the columns that meet the threshold; sum the others into an "Other" column ##
    chosen_cols = [curr for curr in range(len(pivoted_agg)) if pivoted_agg[curr] > cutoff_relative]
    chosen_cols_other = [curr for curr in range(len(pivoted_agg)) if pivoted_agg[curr] <= cutoff_relative]
    pivoted_agg = list(pivoted.iloc[:,chosen_cols_other].sum(axis = 1, skipna = True))
    
    ## Modify the output dataframe accordingly ##
    pivoted = pivoted.iloc[:,chosen_cols]
    pivoted["Other"] = pivoted_agg
    
    return pivoted

def makeConcatFrame(curr_df, new_df, level, sample_name, use_counts):
    new_df = pd.DataFrame(new_df)
    if new_df.empty:
        return curr_df
    if use_counts == 1:
        new_df = pd.DataFrame(new_df.reset_index())
        new_df.columns = [level,"Counts","NumTranscripts","GroupedTranscripts"]
    else:
        new_df = pd.DataFrame(new_df)
        new_df.columns = [level,"NumTranscripts","GroupedTranscripts"]
    
    new_df["Sample"] = sample_name
    return pd.concat([curr_df, new_df], sort=True)

def visualize_all_results(out_prefix, out_dir, est_dir, samples_dir, prot_extension, nucle_extension, use_counts, 
                          rerun, core = False):
    results_frame = dict()
    results_counts_dir = os.path.join(out_dir, "taxonomy_counts")
    results_viz_dir = os.path.join(out_dir, "taxonomy_visualization")
    if core:
        results_counts_dir = os.path.join(out_dir, "core_taxonomy_counts")
        results_viz_dir = os.path.join(out_dir, "core_taxonomy_visualization")

    ### READ IN RESULTS FILES FROM MET DIR THAT FIT SAMPLE SPEC FROM CONFIG ###
    samples = os.listdir(samples_dir)
    print(samples, flush=True)
    good_samples = 0
    for s in samples:
        file_name = ".".join(s.split(".")[0:-1]) + "-estimated-taxonomy.out"
        if (prot_extension in s.split(".")[-1]) | (nucle_extension in s.split(".")[-1]):
            if (not os.path.isfile(os.path.join(est_dir, file_name))):
                print("One of the files, " + s + ", in the sample directory did not complete successfully.")
                if (not core):
                    sys.exit(1)
            else:
                results_frame[file_name] = pd.read_csv(os.path.join(est_dir, file_name), sep = "\t", index_col=0)
        good_samples = good_samples + 1
            
    if (good_samples == 0) & (not core):
        print("No taxonomic estimation files found! Exiting...")
        sys.exit(1)

    list_results = dict()
    frame_results = dict()

    # characterizing by major classes 
    for curr in results_frame.keys():
        list_results[curr], frame_results[curr] = stripClassifData(results_frame[curr], use_counts)
        print(frame_results[curr]["species"].head())

    counts_all = dict()
    level_hierarchy = ['supergroup','division','class','order','family','genus','species']
    for l in level_hierarchy:
        counts_all[l] = pd.DataFrame(columns = [l.capitalize(),"NumTranscripts","GroupedTranscripts","Sample"])

    for curr in results_frame.keys():
        sample_name = curr #curr.split("-")[0]
        for l in level_hierarchy:
            curr_df = counts_all[l]
            new_df = frame_results[curr][l]
            counts_all[l] = makeConcatFrame(curr_df, new_df, l.capitalize(), sample_name, use_counts)

    for l in level_hierarchy:
        ### SAVE THE CSVs OF THE DATA ###
        prefix = out_prefix
        os.system("mkdir -p " + results_counts_dir)
        counts_all[l].to_csv(os.path.join(results_counts_dir, prefix + "_all_" + l + "_counts.csv"))
        
        if (not os.path.isfile(os.path.join(results_counts_dir, prefix + "_all_" + l + "_counts.csv"))):
            print("Taxonomy counts were not successfully generated. Check log for details.")
            sys.exit(1)

        ### INITIALIZE VARIABLES FOR LOOP ###
        Curr_Variable = l.capitalize()
        cutoff_counts = 100000
        cutoff_transcripts = 100

        ### GRAB DATAFRAME FROM SAVED DATA ###
        curr_df_start = counts_all[l].reset_index()
        if Curr_Variable in curr_df_start:
            curr_df_start["OfInterest"] = curr_df_start[Curr_Variable]
        else:
            curr_df_start["OfInterest"] = []

        sns.set()

        ## CREATE AGGREGATED COUNTS BY SAMPLE ##
        curr_df_summed = curr_df_start.groupby("Sample")["NumTranscripts"].agg(AllCts='sum')
        curr_df_summed = curr_df_start.join(curr_df_summed,how="left",on="Sample")

        ## CALCULATE RELATIVE COUNTS ## 
        curr_df_summed["Rel_Counts"] = curr_df_summed["NumTranscripts"] / curr_df_summed["AllCts"]
        curr_df_plot = curr_df_summed[["OfInterest","Sample","Rel_Counts"]]
        pivoted = curr_df_plot.pivot(index='Sample', columns='OfInterest', values='Rel_Counts')

        pivoted_agg = list(pivoted.sum(axis = 0, skipna = True))
        chosen_cols = [curr for curr in range(len(pivoted_agg)) if pivoted_agg[curr] > 0.1]
        pivoted = pivoted.iloc[:,chosen_cols]

        c25 = ["dodgerblue2", "#E31A1C",\
          "green4",\
          "#6A3D9A",\
          "#FF7F00",\
          "black", "gold1",\
          "skyblue2", "#FB9A99",\
          "palegreen2",\
          "#CAB2D6",\
          "#FDBF6F",\
          "gray70", "khaki2",\
          "maroon", "orchid1", "deeppink1", "blue1", "steelblue4",\
          "darkturquoise", "green1", "yellow4", "yellow3",\
          "darkorange4", "brown"]

        fig = plt.figure(figsize=(15,7.5))
        if len(set(curr_df_start["OfInterest"])) == 0:
            continue
        sns_palette = sns.palplot(sns.color_palette("Set1", n_colors=len(set(curr_df_start["OfInterest"]))))

        ### CREATE PLOTS ###
        if use_counts == 0:
            fig = plt.figure(figsize=(15,7.5))
            fig.set_facecolor('white')
            pivoted = createPlotDataFrame(curr_df_start, cutoff_relative = 0.05, transcript_or_counts="NumTranscripts")
            pivoted.plot(kind='bar', stacked=True, color = sns_palette)
            plt.tight_layout()
            os.system("mkdir -p " + results_viz_dir)
            plt.savefig(os.path.join(results_viz_dir, l + '_transcripts.png'),dpi=100)
            plt.show()
            plt.close()
        else:
            f, (ax1, ax2) = plt.subplots(1, 2, sharey=True, figsize=(15,7.5))
            ax1.set_facecolor('white')
            ax2.set_facecolor('white')
            pivoted = createPlotDataFrame(curr_df_start, cutoff_relative = 0.05, transcript_or_counts="NumTranscripts")
            pivoted.plot(kind='bar', stacked=True, width=1, color = sns_palette, title="Transcripts", ax = ax1)
            pivoted = createPlotDataFrame(curr_df_start, cutoff_relative = 0.05, transcript_or_counts="Counts")
            pivoted.plot(kind='bar', stacked=True, width=1, color = sns_palette, title="Counts", ax = ax2)
            plt.tight_layout()
            os.system("mkdir -p " + results_viz_dir)
            plt.savefig(os.path.join(results_viz_dir, l + '_counts_and_transcripts.png'),dpi=100)
            plt.show()
            plt.close()



#!/usr/bin/env python
import ujson
import pandas as pd
import yaml
import chardet
import argparse
import numpy as np
import os

level_dict = {'class':['supergroup','division','class'], 'order':['supergroup','division','class', 'order'], 'family':['supergroup','division','class', 'order', 'family'], 'genus': ['supergroup','division','class', 'order', 'family','genus'], 'species':['supergroup','division','class', 'order', 'family','genus', 'species']}
levels = ['supergroup','division','class','order','family','genus','species']

# bug with splitting based on the new taxonomy estimation
def split_taxonomy(protein_estimate):
    outdf = protein_estimate.copy()
    split_tax_nan = pd.DataFrame(protein_estimate.full_classification.str.split('; '))
    new_col_dict = dict()
    for l in levels:
        new_col_dict[str(l)] = [""] * len(outdf.index)
    for cl in range(len(split_tax_nan.index)):
        lineage = list(split_tax_nan.loc[:,'full_classification'])[cl]
        for i,l in enumerate(levels):
            if isinstance(lineage,list):
                if (len(lineage)>i):
                    new_col_dict[l][cl] = lineage[i]
                else:
                    new_col_dict[l][cl] = np.nan
            else:
                new_col_dict[l][cl] = np.nan
    for l in levels:
        outdf[str(l)] = new_col_dict[l]
    return(outdf)

def create_tax_dictionary(split_taxonomy_df):
    tax_dict = {}
    total_len=len(split_taxonomy_df)
    for l in levels:
        tax_dict[l]={}
        column_sum = split_taxonomy_df.groupby(l)[l].sum()
        column_count = split_taxonomy_df.groupby(l)[l].count()
        norm_count=column_count/total_len
        tax_dict[l]=norm_count
    return(tax_dict)

def get_max_levels(tax_dict):
    max_df = pd.DataFrame(index=levels, columns=['max_taxa','percent_id'])
    for key in tax_dict:
        highest_tax = tax_dict[key][tax_dict[key]==tax_dict[key].max()].index
        max_val = tax_dict[key].max()
        if len(highest_tax)>0:
            max_df.loc[key]=highest_tax[0], max_val
        else:
            max_df.loc[key]=np.nan, max_val
    return(max_df)

def magStats(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument('--estimated-taxonomy-file')
    parser.add_argument('--out-prefix')
    parser.add_argument('--outdir')
    parser.add_argument('--max-out-dir')
    if args != None:
        args = parser.parse_args(args)
    else:
        args = parser.parse_args()
    os.system("mkdir -p " + args.max_out_dir)
    os.system("mkdir -p " + args.outdir)
    estimated_tax = pd.read_csv(args.estimated_taxonomy_file, sep='\t', index_col=0)
    split_taxonomy_df = split_taxonomy(estimated_tax)
    tax_dict = create_tax_dictionary(split_taxonomy_df)
    max_df = get_max_levels(tax_dict)
    if not os.path.exists(args.outdir):
        try:
            os.mkdir(args.outdir)
        except:
            pass
    if not os.path.exists(args.max_out_dir):
        try:
            os.mkdir(args.max_out_dir)
        except:
            pass
    for l in levels:
        tax_dict[l].to_csv(os.path.join(args.outdir, args.out_prefix + '.' + l), header=False, sep='\t')
    max_df.to_csv(os.path.join(args.max_out_dir, args.out_prefix + '-max-level.csv'), sep='\t')
    return 0
    
if __name__ == "__main__":
    magStats()


import os
import sys
import subprocess
import multiprocessing
from joblib import Parallel, delayed
import shutil
import pathlib
import pandas as pd
import math

import EUKulele
from EUKulele.tax_placement import place_taxonomy
from EUKulele.visualize_results import visualize_all_results

from scripts.mag_stats import magStats

MEM_AVAIL_GB = 0
while MEM_AVAIL_GB == 0:
    try:
        os.system("free -m > free.csv")
        MEM_AVAIL_GB = pd.read_csv("free.csv", sep = "\s+").free[0] / 10**3
    except:
        pass
    
# 25 GB memory per GB file size
def calc_max_jobs(size_in_bytes = 2147483648):
    size_in_gb = size_in_bytes / (1024*1024*1024)
    if size_in_gb == 0:
        size_in_gb = 0.01
    MAX_JOBS = math.floor(MEM_AVAIL_GB / (25 * size_in_gb)) #48)
    if MAX_JOBS == 0:
        MAX_JOBS = 1
    return MAX_JOBS
        
MAX_JOBS = calc_max_jobs()

# For DIAMOND: The program can be expected to use roughly six times this number of memory (in GB). 
# So for the default value of -b2.0, the memory usage will be about 12 GB.
# So we want alignment to have -b6.0

def manageEukulele(piece, mets_or_mags = "", samples = [], database_dir = "", 
                   output_dir = "", ref_fasta = "", alignment_choice = "diamond", 
                   rerun_rules = False, cutoff_file = "", sample_dir = "", nt_ext = "", pep_ext = "",
                   consensus_cutoff = 0.75, tax_tab = "", prot_tab = "", use_salmon_counts = False,
                   names_to_reads = "", alignment_res = "", filter_metric = "evalue", 
                   run_transdecoder = False, transdecoder_orf_size = 100):
    
    """
    This function diverts management tasks to the below helper functions.
    """
    
    if piece == "setup_eukulele":
        setupEukulele(output_dir)
    elif piece == "setup_databases":
        createAlignmentDatabase(ref_fasta, rerun_rules, alignment_choice, database_dir)
    elif piece == "get_samples":
        return getSamples(mets_or_mags, sample_dir, nt_ext, pep_ext)
    elif piece == "transdecode":
        if mets_or_mags == "mets":
            manageTrandecode(samples, output_dir, rerun_rules, sample_dir,
                     mets_or_mags = "mets", transdecoder_orf_size = 100,
                     nt_ext = "." + nt_ext.strip('.'), pep_ext = "." + pep_ext.strip('.'),
                     run_transdecoder = run_transdecoder)
    elif piece == "align_to_db":
        return manageAlignment(alignment_choice, samples, filter_metric, output_dir, ref_fasta, 
                        mets_or_mags, database_dir, sample_dir, rerun_rules, nt_ext, pep_ext, core = "full")
    elif piece == "estimate_taxonomy":
        manageTaxEstimation(output_dir, mets_or_mags, tax_tab, cutoff_file, consensus_cutoff,
                            prot_tab, use_salmon_counts, names_to_reads, alignment_res,
                            rerun_rules, samples, sample_dir, pep_ext, nt_ext)
    elif piece == "visualize_taxonomy":
        manageTaxVisualization(output_dir, mets_or_mags, sample_dir, pep_ext, nt_ext, 
                               use_salmon_counts, rerun_rules)
    elif piece == "assign_taxonomy":
        manageTaxAssignment(samples, mets_or_mags, output_dir, sample_dir, pep_ext, core = False)
    elif piece == "core_align_to_db":
        alignment_res = manageAlignment(alignment_choice, samples, filter_metric, output_dir, ref_fasta, 
                        mets_or_mags, database_dir, sample_dir, rerun_rules, nt_ext, pep_ext, core = "core")
        alignment_res = [curr for curr in alignment_res if curr != ""]
        return alignment_res
    elif piece == "core_estimate_taxonomy":
        manageCoreTaxEstimation(output_dir, mets_or_mags, tax_tab, cutoff_file, consensus_cutoff,
                            prot_tab, use_salmon_counts, names_to_reads, alignment_res,
                            rerun_rules, samples, sample_dir, pep_ext, nt_ext)
    elif piece == "core_visualize_taxonomy":
        manageCoreTaxVisualization(output_dir, mets_or_mags, sample_dir, pep_ext, nt_ext, 
                               use_salmon_counts, rerun_rules, core = True)
    elif piece == "core_assign_taxonomy":
        manageTaxAssignment(samples, mets_or_mags, output_dir, sample_dir, pep_ext, core = True)
    else:
        print("Not a supported management function.")
        sys.exit(1)
        
             
def getSamples(mets_or_mags, sample_dir, nt_ext, pep_ext):
    """
    Get the names of the metagenomic or metatranscriptomic samples from the provided folder.
    """
    
    if (mets_or_mags == "mets"):
        samples = [".".join(curr.split(".")[0:-1]) for curr in os.listdir(sample_dir) if curr.split(".")[-1] == nt_ext]
        if len(samples) == 0:
            print("No samples found in sample directory with specified nucleotide extension.")
            sys.exit(1)
    else:
        samples = [".".join(curr.split(".")[0:-1]) for curr in os.listdir(sample_dir) if curr.split(".")[-1] == pep_ext]
        if len(samples) == 0:
            print("No samples found in sample directory with specified peptide extension.")
            sys.exit(1)
            
    return samples
            

def transdecodeToPeptide(sample_name, output_dir, rerun_rules, sample_dir, 
                         mets_or_mags = "mets", transdecoder_orf_size = 100,
                         nt_ext = ".fasta", pep_ext = ".faa", run_transdecoder = False):
    """
    Use TransDecoder to convert input nucleotide metatranscriptomic sequences to peptide sequences.
    """
    
    print("Running TransDecoder for sample " + str(sample_name) + "...", flush = True)
    os.system("mkdir -p " + os.path.join(output_dir, mets_or_mags, "transdecoder"))
    if (os.path.isfile(os.path.join(output_dir, mets_or_mags, 
                                    sample_name + pep_ext))) & (not rerun_rules):
        print("TransDecoder file already detected for sample " + 
              str(sample_name) + "; will not re-run step.", flush = True)
        return 0
    elif (os.path.isfile(os.path.join(sample_dir, sample_name + pep_ext))) & (not rerun_rules):
        print("Protein files detected for sample in sample directory; " +
              "will not TransDecode.", flush = True)
        os.system("cp " + os.path.join(sample_dir, sample_name + pep_ext) + " " +
                  os.path.join(output_dir, mets_or_mags, sample_name + pep_ext))
        return 0
    
    if (not run_transdecoder):
        return 0
    
    TD_log = open(os.path.join("log","transdecoder_longorfs_" + sample_name + ".log"), "w+")
    TD_err = open(os.path.join("log","transdecoder_longorfs_" + sample_name + ".err"), "w+")
    if (not os.path.isfile(os.path.join(sample_dir, sample_name + nt_ext))):
        print("File: " + os.path.join(sample_dir, sample_name + nt_ext) + " was called by TransDecoder and "
              "does not exist. Check for typos.")
        sys.exit(1)
    rc1 = subprocess.Popen(["TransDecoder.LongOrfs", "-t", os.path.join(sample_dir, sample_name + nt_ext),
               "-m", str(transdecoder_orf_size)], stdout = TD_log, stderr = TD_err).wait()
    TD_log.close()
    TD_err.close()
    
    TD_log = open(os.path.join("log","transdecoder_predict_" + sample_name + ".log"), "w+") 
    TD_err = open(os.path.join("log","transdecoder_predict_" + sample_name + ".err"), "w+")
    rc2 = subprocess.Popen(["TransDecoder.Predict", "-t", os.path.join(sample_dir, sample_name + nt_ext),
               "--no_refine_starts"], stdout = TD_log, stderr = TD_err).wait()
    #rc2 = p2.returncode
    TD_log.close()
    TD_err.close()
    
    if (rc1 + rc2) != 0:
        print("TransDecoder did not complete successfully for sample " + 
              str(sample_name) + ". Check log/ folder for details.")
        sys.exit(1)
        
    merged_name = sample_name + nt_ext
    
    os.system("mkdir -p " + os.path.join(output_dir, mets_or_mags))
    os.system("mkdir -p " + os.path.join(output_dir, mets_or_mags, "transdecoder"))
    
    os.replace(merged_name + ".transdecoder.pep", os.path.join(output_dir, mets_or_mags, 
                                                               sample_name + pep_ext))
    os.replace(merged_name + ".transdecoder.cds", os.path.join(output_dir, mets_or_mags, 
                                                               "transdecoder", sample_name + 
                                                               ".fasta.transdecoder.cds"))
    os.replace(merged_name + ".transdecoder.gff3", os.path.join(output_dir, mets_or_mags, 
                                                                "transdecoder", sample_name + 
                                                                ".fasta.transdecoder.gff3"))
    os.replace(merged_name + ".transdecoder.bed", os.path.join(output_dir, mets_or_mags, 
                                                               "transdecoder", sample_name + 
                                                               ".fasta.transdecoder.bed"))
    #shutil.rmtree
    os.system("rm -rf " + merged_name + "*.transdecoder_dir*")
    return rc1 + rc2
    
def manageTrandecode(met_samples, output_dir, rerun_rules, sample_dir, 
                     mets_or_mags = "mets", transdecoder_orf_size = 100,
                     nt_ext = "fasta", pep_ext = ".faa", run_transdecoder = False):
    """
    Now for some TransDecoding - a manager for TransDecoder steps.
    """
    
    print("Running TransDecoder for MET samples...", flush = True)
    MAX_JOBS = max([calc_max_jobs(pathlib.Path(os.path.join(sample_dir, sample + nt_ext)).stat().st_size) 
                    for sample in met_samples])
    n_jobs_align = min(multiprocessing.cpu_count(), len(met_samples), MAX_JOBS)
    transdecoder_res = Parallel(n_jobs=n_jobs_align)(delayed(transdecodeToPeptide)(sample_name, output_dir, 
                                                                                   rerun_rules, sample_dir, 
                         mets_or_mags = "mets", transdecoder_orf_size = 100,
                         nt_ext = nt_ext, pep_ext = pep_ext, 
                         run_transdecoder = run_transdecoder) for sample_name in met_samples)
    all_codes = sum(transdecoder_res)
    os.system("rm -f pipeliner*")
    if all_codes > 0:
        print("TransDecoder did not complete successfully; check log folder for details.")
        sys.exit(1)
    #rcodes = [os.remove(curr) for curr in glob.glob("pipeliner*")]
              
def setupEukulele(output_dir):
    print("Setting things up...")
    os.system("mkdir -p " + output_dir)
    os.system("mkdir -p log")

    ## Download software dependencies
    rc1 = os.system("install_dependencies.sh references_bins/ " + 
                    "1> log/dependency_log.txt 2> log/dependency_err.txt")
    sys.path.append("references_bins/")
    os.system("echo $PATH > path_test.txt")
    if rc1 != 0:
        print("Could not successfully install all external dependent software.\n" + 
              "Check DIAMOND, BLAST, BUSCO, and TransDecoder installation.")
        return 1
    return 0

def manageAlignment(alignment_choice, samples, filter_metric, output_dir, ref_fasta,
                    mets_or_mags, database_dir, sample_dir, rerun_rules, nt_ext, pep_ext, core = "full"):
    """
    Manage the multithreaded management of aligning to either BLAST or DIAMOND database.
    """
    
    print("Aligning to reference database...")
    if mets_or_mags == "mets":
        fastas = [os.path.join(output_dir, mets_or_mags, sample + "." + pep_ext) if 
                  os.path.isfile(os.path.join(output_dir, mets_or_mags, sample + "." + pep_ext)) else
                  os.path.join(sample_dir, sample + "." + nt_ext)
                  for sample in samples]
    else:
        fastas = [os.path.join(sample_dir, sample + "." + pep_ext) for sample in samples]
        
    MAX_JOBS = max([calc_max_jobs(pathlib.Path(sample).stat().st_size) for sample in fastas])
    n_jobs_align = min(multiprocessing.cpu_count(), len(samples), MAX_JOBS)
    print(MAX_JOBS)
    alignment_res = Parallel(n_jobs=n_jobs_align, prefer="threads")(delayed(alignToDatabase)(alignment_choice,
                                                                                               sample_name, filter_metric, 
                                                                                               output_dir, ref_fasta, 
                                                                                               mets_or_mags, database_dir, 
                                                                                               sample_dir, rerun_rules, nt_ext, 
                                                                                               pep_ext, core = core) \
                                                                    for sample_name in samples)
    #alignment_res = []
    #for sample_name in samples:
    #    alignment_res.append(alignToDatabase(alignment_choice, sample_name, filter_metric, output_dir, ref_fasta, 
    #                     mets_or_mags, database_dir, sample_dir, rerun_rules, nt_ext, pep_ext))
    
    if any([((curr == None) | (curr == 1)) for curr in alignment_res]):
        print("Alignment did not complete successfully.")
        sys.exit(1)
        
    return alignment_res

def createAlignmentDatabase(ref_fasta, rerun_rules, alignment_choice="diamond", database_dir=""):
    """
    Creates a database from the provided reference fasta file and reference database,
    whether or not it has been autogenerated.
    """
    
    rc2 = 0
              
    output_log = os.path.join("log", "alignment_out.log")
    error_log = os.path.join("log", "alignment_err.log")
    if alignment_choice == "diamond":
        align_db = os.path.join(database_dir, "diamond", ref_fasta.strip('.fa') + '.dmnd')
        if (not os.path.isfile(align_db)) | (rerun_rules):
            ## DIAMOND database creation ##
            os.system("mkdir -p " + os.path.join(database_dir, "diamond"))
            db = os.path.join(database_dir, "diamond", ref_fasta.strip('.fa'))
            rc2 = os.system("diamond makedb --in " + os.path.join(database_dir, ref_fasta) + " --db " + db + 
                            " 1> " + output_log + " 2> " + error_log)
        else:
            print("Diamond database file already created; will not re-create database.", flush = True)
    else:
        db = os.path.join(database_dir, "blast", ref_fasta.strip('.fa'), "database")
        os.system("mkdir -p " + db)
        db_type = "prot"
        blast_version = 5
        database = ref_fasta.strip('.')
        # makeblastdb -in tests/aux_data/mmetsp/sample_ref_MAG/reference.pep.fa -parse_seqids -title referencefa -dbtype prot -out tests/aux_data/mmetsp/sample_ref_MAG/blast/reference.pep/database
        rc2 = os.system("makeblastdb -in " + os.path.join(database_dir, ref_fasta) + " -parse_seqids -title " + database + 
                        " -dbtype " + db_type + " -out " + db + " 1> " + output_log + " 2> " + 
                        error_log)
    return rc2
 
def alignToDatabase(alignment_choice, sample_name, filter_metric, output_dir, ref_fasta,
                      mets_or_mags, database_dir, sample_dir, rerun_rules, nt_ext, pep_ext, core = "full"):
    """
    Align the samples against the created database.
    """
    
    print("Aligning sample " + sample_name + "...")
    if alignment_choice == "diamond":
        os.system("mkdir -p " + os.path.join(output_dir, mets_or_mags + "_" + core, "diamond"))
        diamond_out = os.path.join(output_dir, mets_or_mags + "_" + core, "diamond", sample_name + ".diamond.out")
        if (os.path.isfile(diamond_out)):
            if (pathlib.Path(diamond_out).stat().st_size != 0) & (not rerun_rules):
                print("Diamond alignment file already detected; will not re-run step.")
                return diamond_out
        
        align_db = os.path.join(database_dir, "diamond", ref_fasta.strip('.fa') + '.dmnd')
        alignment_method = "blastp"
        if mets_or_mags == "mets":
            if os.path.isfile(os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext)):
                fasta = os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext) 
            else:
                fasta = os.path.join(sample_dir, sample_name + "." + nt_ext)
                alignment_method = "blastx"
        elif core == "full":
            fasta = os.path.join(sample_dir, sample_name + "." + pep_ext)
        elif core == "core":
            # now concatenate the BUSCO output
            fasta = os.path.join(output_dir, sample_name + "_busco" + "." + pep_ext)
            os.system(" ".join(["concatenate_busco.sh", sample_name, fasta, output_dir]))
            if not os.path.isfile(fasta):
                print("No BUSCO matches found for sample: " + sample_name)
                return ""
            
        other = "--outfmt 6 -k 100 -e 1e-5"
        outfmt = 6
        k = 100
        e = 1e-5
        bitscore = 50
        pid_cutoff = 75
        diamond_log = open(os.path.join("log",core + "_diamond_align_" + sample_name + ".log"), "w+")
        diamond_err = open(os.path.join("log",core + "_diamond_align_" + sample_name + ".err"), "w+")
        if filter_metric == "bitscore":
            rc1 = subprocess.Popen(["diamond", alignment_method, "--db", align_db, "-q", fasta, "-o", 
                                   diamond_out, "--outfmt", str(outfmt), "-k", str(k), "--min-score", 
                                   str(bitscore), '-b3.0'], stdout = diamond_log, stderr = diamond_err).wait()
            print("Diamond process exited.", flush = True)
        elif filter_metric == "pid":
            rc1 = subprocess.Popen(["diamond", alignment_method, "--db", align_db, "-q", fasta, "-o", 
                                   diamond_out, "--outfmt", str(outfmt), "-k", str(k), "--id", 
                                   str(pid_cutoff), '-b3.0'], stdout = diamond_log, stderr = diamond_err).wait()
            print("Diamond process exited.", flush = True)
        else:
            rc1 = subprocess.Popen(["diamond", alignment_method, "--db", align_db, "-q", fasta, "-o", 
                                   diamond_out, "--outfmt", str(outfmt), "-k", str(k), "-e", 
                                   str(e), '-b3.0'], stdout = diamond_log, stderr = diamond_err).wait()
            # For debugging.
            #, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            #stdout, stderr = p.communicate()
            #rc1 = p.returncode
            #print(stderr)
            #print(stdout)
            print("Diamond process exited.", flush = True)
        if rc1 != 0:
            print("Diamond did not complete successfully.")
            os.system("rm -f " + diamond_out)
            return 1
        return diamond_out
    else:
        blast_out = os.path.join(output_dir, mets_or_mags + "_" + core, "blast", sample_name + ".blast.txt")
        os.system("mkdir -p " + os.path.join(output_dir, mets_or_mags + "_" + core, "blast"))
        if (os.path.isfile(blast_out)) & (not rerun_rules):
            print("BLAST alignment file already detected; will not re-run step.")
            return blast_out
        
        align_db = os.path.join(database_dir, "blast", ref_fasta.strip('.fa'), "database")
        alignment_method = "blastp"
        if mets_or_mags == "mets":
            if os.path.isfile(os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext)):
                fasta = os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext) 
            else:
                fasta = os.path.join(sample_dir, sample_name + "." + nt_ext)
                alignment_method = "blastx"
        elif core == "full":
            fasta = os.path.join(sample_dir, sample_name + "." + pep_ext)
        elif core == "core":
            # now concatenate the BUSCO output
            fasta = os.path.join(output_dir, sample_name + "_busco" + "." + pep_ext)
            os.system(" ".join(["concatenate_busco.sh", sample_name, fasta, output_dir]))
            if not os.path.isfile(fasta):
                print("No BUSCO matches found for sample: " + sample_name)
                return ""
            
        outfmt = 6 # tabular output format
        e = 1e-5
        os.system("export BLASTDB=" + align_db)
        blast_log = open(os.path.join("log","blast_align_" + sample_name + ".log"), "w+")
        blast_err = open(os.path.join("log","blast_align_" + sample_name + ".err"), "w+")
        rc1 = subprocess.Popen([alignment_method, "-query", fasta, "-db", align_db, "-out", 
                              blast_out,"-outfmt",str(outfmt),"-evalue", str(e)],
                             stdout = blast_log, stderr = blast_err).wait()
        if rc1 != 0:
            print("BLAST did not complete successfully.")
            return 1
        return blast_out
    
    
def manageTaxEstimation(output_dir, mets_or_mags, tax_tab, cutoff_file, consensus_cutoff,
                        prot_tab, use_salmon_counts, names_to_reads, alignment_res,
                        rerun_rules, samples, sample_dir, pep_ext, nt_ext):
    print("Performing taxonomic estimation steps...", flush=True)
    os.system("mkdir -p " + os.path.join(output_dir, "taxonomy_estimation"))
    outfiles = [os.path.join(output_dir, "taxonomy_estimation", samp + "-estimated-taxonomy.out") for samp in samples]
    
    if mets_or_mags == "mets":
        fastas = []
        for sample in samples:
            if os.path.isfile(os.path.join(output_dir, mets_or_mags, sample + "." + pep_ext)):
                fastas.append(os.path.join(output_dir, mets_or_mags, sample + "." + pep_ext))
            else:
                fastas.append(os.path.join(sample_dir, sample + "." + nt_ext))
    else:
        fastas = [os.path.join(sample_dir, sample + "." + pep_ext) for sample in samples]
        
    MAX_JOBS = max([calc_max_jobs(pathlib.Path(sample).stat().st_size) for sample in fastas])
    n_jobs_align = min(multiprocessing.cpu_count(), len(alignment_res), MAX_JOBS)
    for t in range(len(alignment_res)): 
        try:
            sys.stdout = open(os.path.join("log", "tax_est_" + alignment_res[t].split("/")[-1].split(".")[0] + ".out"), "w")
            sys.stderr = open(os.path.join("log", "tax_est_" + alignment_res[t].split("/")[-1].split(".")[0] + ".err"), "w")
            curr_out = place_taxonomy(tax_tab, cutoff_file, consensus_cutoff,\
                                                    prot_tab, use_salmon_counts, names_to_reads,\
                                                    alignment_res[t], outfiles[t], rerun_rules)
        except:
            print("Taxonomic estimation for core genes did not complete successfully. Check log file for details.")
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__
        
def manageCoreTaxEstimation(output_dir, mets_or_mags, tax_tab, cutoff_file, consensus_cutoff,
                            prot_tab, use_salmon_counts, names_to_reads, alignment_res,
                            rerun_rules, samples, sample_dir, pep_ext, nt_ext):
    print("Performing taxonomic estimation steps...", flush=True)
    os.system("mkdir -p " + os.path.join(output_dir, "core_taxonomy_estimation"))
    outfiles = [os.path.join(output_dir, "core_taxonomy_estimation", samp + "-estimated-taxonomy.out") for samp in samples]
    
    if mets_or_mags == "mets":
        fastas = []
        for sample in samples:
            if os.path.isfile(os.path.join(output_dir, mets_or_mags, sample + "." + pep_ext)):
                fastas.append(os.path.join(output_dir, mets_or_mags, sample + "." + pep_ext))
            else:
                fastas.append(os.path.join(sample_dir, sample + "." + nt_ext))
    else:
        fastas = [os.path.join(sample_dir, sample + "." + pep_ext) for sample in samples]
        
    MAX_JOBS = max([calc_max_jobs(pathlib.Path(sample).stat().st_size) for sample in fastas])
    n_jobs_align = min(multiprocessing.cpu_count(), len(alignment_res), MAX_JOBS)
    for t in range(len(alignment_res)): 
        try:
            sys.stdout = open(os.path.join("log", "core_tax_est_" + 
                                           alignment_res[t].split("/")[-1].split(".")[0] + ".out"), "w")
            sys.stderr = open(os.path.join("log", "core_tax_est_" + 
                                           alignment_res[t].split("/")[-1].split(".")[0] + ".err"), "w")
            curr_out = place_taxonomy(tax_tab, cutoff_file, consensus_cutoff,\
                                                    prot_tab, use_salmon_counts, names_to_reads,\
                                                    alignment_res[t], outfiles[t], rerun_rules)
        except:
            print("Taxonomic estimation for core genes did not complete successfully. Check log file for details.")
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__
        
def manageTaxVisualization(output_dir, mets_or_mags, sample_dir, pep_ext, nt_ext, use_salmon_counts, rerun_rules):
    print("Performing taxonomic visualization steps...", flush=True)
    out_prefix = output_dir.split("/")[-1]
    sys.stdout = open(os.path.join("log", "tax_vis.out"), "w")
    sys.stderr = open(os.path.join("log", "tax_vis.err"), "w")
    visualize_all_results(out_prefix, output_dir, os.path.join(output_dir, "taxonomy_estimation"), 
                          sample_dir, pep_ext, nt_ext, use_salmon_counts, rerun_rules)
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__
    
def manageCoreTaxVisualization(output_dir, mets_or_mags, sample_dir, pep_ext, nt_ext, use_salmon_counts, 
                               rerun_rules, core = False):
    print("Performing taxonomic visualization steps...", flush=True)
    out_prefix = output_dir.split("/")[-1]
    sys.stdout = open(os.path.join("log", "core_tax_vis.out"), "w")
    sys.stderr = open(os.path.join("log", "core_tax_vis.err"), "w")
    visualize_all_results(out_prefix, output_dir, os.path.join(output_dir, "core_taxonomy_estimation"), 
                          sample_dir, pep_ext, nt_ext, use_salmon_counts, rerun_rules, core)
    #except:
    #    print("Taxonomic visualization of core genes did not complete successfully. Check log files for details.")
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__

def manageTaxAssignment(samples, mets_or_mags, output_dir, sample_dir, pep_ext, core = False):
    if mets_or_mags == "mags":
        print("Performing taxonomic assignment steps...", flush=True)
        MAX_JOBS = max([calc_max_jobs(pathlib.Path(os.path.join(sample_dir, sample + "." + pep_ext)).stat().st_size) 
                        for sample in samples])
        n_jobs_viz = min(multiprocessing.cpu_count(), len(samples), MAX_JOBS)
        try:
            if core:
                assign_res = Parallel(n_jobs=n_jobs_viz, prefer="threads")(delayed(assignTaxonomy)(samp, output_dir, 
                                                                                                   "core_taxonomy_estimation",
                                                                                                mets_or_mags, core = True) \
                                                                           for samp in samples)
            else:
                assign_res = Parallel(n_jobs=n_jobs_viz, prefer="threads")(delayed(assignTaxonomy)(samp, output_dir,
                                                                                                   "taxonomy_estimation",
                                                                                                mets_or_mags, core = False) \
                                                                           for samp in samples)
        except:
            print("Taxonomic assignment did not complete successfully. Check log files for details.")
            sys.exit(1)
        
        if sum(assign_res) != 0:
            print("Taxonomic assignment did not complete successfully. Check log files for details.")
            sys.exit(1)
            
def assignTaxonomy(sample_name, output_dir, est_dir, mets_or_mags, core = False):
    taxfile = os.path.join(output_dir, est_dir, sample_name + "-estimated-taxonomy.out")
    levels_directory = os.path.join(output_dir, "levels_mags")
    max_dir = os.path.join(output_dir, "max_level_mags")
    if core:
        levels_directory = os.path.join(output_dir, "core_levels_mags")
        max_dir = os.path.join(output_dir, "core_max_level_mags")
        
    error_log = os.path.join("log", "_".join(est_dir.split("_")[0:1]) + "_assign_" + sample_name + ".err")
    out_log = os.path.join("log", "_".join(est_dir.split("_")[0:1]) + "_assign_" + sample_name + ".out")
    
    sys.stdout = open(out_log, "w")
    sys.stderr = open(error_log, "w")
    try:
        rc = magStats(["--estimated-taxonomy-file",taxfile,
                       "--out-prefix",sample_name,"--outdir",
                       levels_directory,"--max-out-dir",max_dir])
    except:
        print("Taxonomic assignment did not complete successfully for sample " + str(sample_name) +
              ". Check log for details.")
        sys.exit(1)
    sys.stdout = sys.__stdout__
    sys.stderr = sys.__stderr__
    return rc



import pandas as pd
import glob
import os
import yaml
import sys

def namesToReads(reference_dir, names_to_reads, salmon_dir):

    if os.path.isfile(os.path.join(reference_dir,names_to_reads)):
        print("Salmon reads file previously created; new file will not be created from Salmon directory.")
        sys.exit(0)

    folder_names = glob.glob(os.path.join(salmon_dir,'*quant*'))
    files_salmon = [os.path.join(curr,"quant.sf") for curr in folder_names]

    transcript_names = []
    transcript_counts = []
    sample_names = []

    for curr_ind in range(len(folder_names)):
        curr_salmon = pd.read_csv(files_salmon[curr_ind], sep = "\t")
        transcript_names.extend([name_curr.split(".")[0] for name_curr in curr_salmon["Name"]])
        transcript_counts.extend(curr_salmon["NumReads"])
        sample_names.extend([folder_names[curr_ind].split("_")[-2]] * len(curr_salmon.index))

    names_to_reads = pd.DataFrame({"TranscriptNames": transcript_names, "NumReads": transcript_counts, 
                                   "SampleName": sample_names})
    if ".csv" in names_to_reads:
        names_to_reads.to_csv(path_or_buf = os.path.join(reference_dir,names_to_reads), sep = "\t")
    else:
        names_to_reads.to_csv(path_or_buf = os.path.join(reference_dir,"namestoreads.csv"), sep = "\t")
        names_to_reads = "namestoreads.csv"
    
    return names_to_reads


#!/usr/bin/env python3

"""
USAGE: 

    python create-protein-table.py --infile_peptide input.fa --infile_taxonomy tax.txt --outfile_json protein-species-map.json --output output_table.txt [--delim delimiter --column column] 

Generates table useable for taxonomic placment with EUKulele. If no delimiter is provided, default is '/'. If no column header is provided, default is SOURCE_ID. 

python EUKulele/scripts/create_protein_table.py --infile_peptide EUKulele/tests/aux_data/mmetsp/reference-pep-trunc.pep.faa --infile_taxonomy  EUKulele/tests/aux_data/mmetsp/taxonomy-table.txt --outfile_json EUKulele/tests/aux_data/mmetsp/protein-map.json --output EUKulele/tests/aux_data/mmetsp/tax-table.txt --delim "/" --strain_col_id strain_name --taxonomy_col_id taxonomy --column SOURCE_ID

        python /vortexfs1/omics/alexander/akrinos/remodeling/EUKulele/scripts/create_protein_table.py --infile_peptide /vortexfs1/omics/alexander/akrinos/EUKulele-Reference/phylodb_db/reference.pep.fa --infile_taxonomy  /vortexfs1/omics/alexander/akrinos/EUKulele-Reference/phylodb_db/tax-table.txt --outfile_json /vortexfs1/omics/alexander/akrinos/EUKulele-Reference/phylodb_db/prot-map.json --output /vortexfs1/omics/alexander/akrinos/EUKulele-Reference/phylodb_db/taxonomy_table.txt --delim "\t" --strain_col_id strain_name --taxonomy_col_id taxonomy --column 2
"""

from Bio import SeqIO
import sys
import argparse
import json
import pandas as pd

def createProteinTable(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument('--infile_peptide', type = list, nargs='+', required=True) # this should be given as a list of the input files
    parser.add_argument('--infile_taxonomy',  default='') # the original taxonomy file
    parser.add_argument('--outfile_json', default = 'prot-map.json') # the protein json file to be written
    parser.add_argument('--output', default = 'tax-table.txt') # the output taxonomy table (formatted) to be written
    parser.add_argument('--delim',  type=str, default = '/')
    parser.add_argument('--col_source_id',  type=str, default = 'Source_ID') # the column which indicates the name of the strain in the taxonomy file
    parser.add_argument('--taxonomy_col_id',  type=str, default = 'taxonomy') # the column which indicates the taxonomy of the strain in the taxonomy file
    parser.add_argument('--column', type=str, default = 'SOURCE_ID') # can be numeric, zero-indexed, if it's a delimited part of header
    # set to true if there is a column called "taxonomy" that we wish to split
    parser.add_argument('--reformat_tax', dest='reformat', default=False, action='store_true') 
    parser.add_argument('--euk-prot', dest='eukprot', default=False, action='store_true') # eukprot's taxonomy is too unique

    if args != None:
        args = parser.parse_args(args)
    else:
        args = parser.parse_args()
    # if the input is a folder, you also need to add the first token in the underscore-separated name to a dictionary for that
    # one - ultimately we want to know which MMETSP or whatever it came from
    odict = {}
    for curr_pepfile in list(args.infile_peptide):
        print("".join(curr_pepfile))
        pepfile = "".join(curr_pepfile)
        for record in SeqIO.parse(pepfile, "fasta"):
            header = record.description
            rid = record.id.replace(".","N") #record.id.split(".")[0] #record.id.replace(".","N")
            if 't' in args.delim: # why is this tab thing not working otherwise?? even the equality
                tester = "".join(list(str(header))).replace('\t', '    ')
                hlist = tester.split("    ")
            else:
                header = str(header).replace(args.delim, "hello")
                hlist = header.split("hello")
            if len(args.infile_peptide) > 1: # if there is a list of files, use the filename as the ID
                sid = pepfile.split("/")[-1].split("_")[0]
                odict[rid] = sid
            elif args.column.isdigit():
                sid = hlist[int(args.column)]  
                odict[rid] = sid
            else:
                for h in hlist:
                    if args.column in h: #h.startswith(args.column):
                        sid = h.split('=')[1].strip()    
                        odict[rid] = sid
                        break
    tax_file = pd.read_csv(args.infile_taxonomy, sep = "\t", encoding='latin-1')

    if args.reformat:
        colnames_tax = ["Source_ID","Supergroup","Division","Class","Order","Family","Genus","Species"]
        tax_out = pd.DataFrame(columns=colnames_tax)
        for i in range(0,len(tax_file.index)):
            if not args.eukprot:
                curr_row = [tax_file[args.col_source_id][i]] + tax_file[args.taxonomy_col_id][i].split(";")
                if len(curr_row) < (len(colnames_tax)):
                    curr_row = curr_row + [""] * ((len(colnames_tax) + 1) - len(curr_row))
                elif len(curr_row) > (len(colnames_tax)):
                    curr_row = curr_row[0:7] + [curr_row[8]]
                add_series = pd.Series(curr_row, index = colnames_tax)
                tax_out = tax_out.append(add_series, ignore_index=True)
            else:
                curr_row = [tax_file[args.col_source_id][i]] + [""] * 7
                full_taxonomy = tax_file[args.taxonomy_col_id][i].split(";")
                genus_and_species = tax_file["Name_to_Use"][i].replace("_", " ")
                curr_row[1] = full_taxonomy[0] # this is generally the "supergroup" the way we have used it thus far.
                curr_row[2] = tax_file["Supergroup_UniEuk"][i] # I believe this is closest to the division
                curr_row[4] = tax_file["Taxogroup_UniEuk"][i] # I believe this is closest to the order
                curr_row[6] = tax_file["Genus_UniEuk"][i]
                curr_row[7] = genus_and_species
                print(curr_row)
                add_series = pd.Series(curr_row, index = colnames_tax)
                tax_out = tax_out.append(add_series, ignore_index=True)

        tax_file = tax_out
    tax_file.to_csv(args.output,sep="\t")
    with open(args.outfile_json, 'w') as f:
        json.dump(odict, f)
        
    return 0
        
if __name__ == "__main__": 
    createProteinTable()


# Query BUSCO for a given species or functional group. 
# A script that takes a given functional group as input, along with the taxonomic level of that group
# (e.g. Phaeocystis antarctica, species), and then checks for BUSCO completeness among the contigs
# identified as that taxonomic level or lower, also evaluating the number of copies of the BUSCO
# matches to differentiate between multiple strains. # !/ usr/bin/env python3

# os.path.join("EUKulele/tests/aux_data/test_out","test_out")
# python query_busco.py --organism_group Chromera --taxonomic_level genus --output_dir check_dir --fasta_file  --sample_name sample_0 --taxonomy_file_prefix   --tax_table " + TAX_TAB + " --busco_out " + busco_table

import pandas as pd
import numpy as np
import os
import sys
import argparse
import chardet
import glob
import multiprocessing
from joblib import Parallel, delayed

__author__ = "Arianna Krinos, Harriet Alexander"
__copyright__ = "EUKulele"
__license__ = "MIT"
__email__ = "akrinos@mit.edu"

level_hierarchy = ['supergroup','division','class','order','family','genus','species']

def evaluate_organism(organism, taxonomy, tax_table, create_fasta, write_transcript_file, busco_out, 
                      taxonomy_file_prefix, busco_threshold, output_dir, sample_name, fasta_file):
    organism_format = organism
    if organism == "":
        print("No organism found")
        return pd.DataFrame(columns = ["Organism","TaxonomicLevel","BuscoCompleteness","NumberCovered",
                                       "CtTwoCopies","CtThreeCopies","CtFourCopies","CtFivePlusCopies",
                                       "PercentageDuplicated"])
    if taxonomy == "species":
        organism_format = " ".join(str(organism).split(";"))
    full_taxonomy = tax_table.loc[[(organism_format in curr) for curr in list(tax_table[taxonomy])],:]
    if len(full_taxonomy.index) < 1:
        print("No taxonomy found for that organism " + str(organism) + " and taxonomic level " + str(taxonomy) + ".")
        return pd.DataFrame(columns = ["Organism","TaxonomicLevel","BuscoCompleteness","NumberCovered",
                                       "CtTwoCopies","CtThreeCopies","CtFourCopies","CtFivePlusCopies",
                                       "PercentageDuplicated"])
        
    curr_level = [ind for ind in range(len(level_hierarchy)) if level_hierarchy[ind] == taxonomy][0]
    max_level = len(level_hierarchy) - 1

    success = 0
    success_level = ""
    busco_scores = []
    levels_out = []
    percent_multiples = []
    number_duplicated = [] 
    number_tripled = []
    number_quadrupled = []
    number_higher_mult = []
    number_covered = []
    busco_out_file = pd.read_csv(busco_out, sep = "\t", comment = "#", names = ["BuscoID","Status","Sequence","Score","Length"])
    select_inds = [ (busco_out_file.Status[curr] == "Complete") | 
                   (busco_out_file.Status[curr] == "Fragmented") | 
                   (busco_out_file.Status[curr] == "Duplicated") for curr in range(len(busco_out_file.index))]
    good_buscos = busco_out_file.loc[select_inds,:]
    good_busco_sequences = [curr.split(".")[0] for curr in list(good_buscos.Sequence)]
    if len(good_busco_sequences) == 0:
        print("No BUSCO matches were made")
        return pd.DataFrame(columns = ["Organism","TaxonomicLevel","BuscoCompleteness","NumberCovered",
                                       "CtTwoCopies","CtThreeCopies","CtFourCopies","CtFivePlusCopies",
                                       "PercentageDuplicated"])
    
    Covered_IDs = list(good_buscos.BuscoID)
    total_buscos = len(set(list(busco_out_file.BuscoID)))
    
    while (curr_level >= 0):

        #### GET THE CURRENT LEVEL OF TAXONOMY FROM THE TAX TABLE FILE ####
        curr_tax_list = set(list(full_taxonomy[level_hierarchy[curr_level]]))
        if len(curr_tax_list) > 1:
            print("More than 1 unique match found; using all matches: " + str(", ".join(curr_tax_list)))
        curr_taxonomy = ";".join(curr_tax_list)
        if (curr_taxonomy == "") | (curr_taxonomy.lower() == "nan"):
            print("No taxonomy found at level " + level_hierarchy[curr_level])
            continue

        #### CREATE A "MOCK TRANSCRIPTOME" BY PULLING BY TAXONOMIC LEVEL ####
        taxonomy_file = pd.read_csv(taxonomy_file_prefix + "_all_" + 
                                    str(level_hierarchy[curr_level]) + 
                                    "_counts.csv", sep=",",header=0)
        taxonomy_file = taxonomy_file.loc[[tax in curr_taxonomy for tax in list(taxonomy_file[level_hierarchy[curr_level].capitalize()])],:]
        transcripts_to_search = list(taxonomy_file["GroupedTranscripts"])
        transcripts_to_search_sep = []
        for transcript_name in transcripts_to_search:
            transcripts_to_search_sep.extend([curr.split(".")[0] for curr in transcript_name.split(";")])

        set_transcripts_to_search = set(transcripts_to_search_sep)
        good_busco_sequences_list = list(good_busco_sequences)
        BUSCOs_covered_all = [Covered_IDs[curr] for curr in range(len(good_busco_sequences_list)) if \
                              good_busco_sequences_list[curr] in list(set_transcripts_to_search)]
        BUSCOs_covered = set(BUSCOs_covered_all)
        number_appearences = [BUSCOs_covered_all.count(curr_busco) for curr_busco in list(BUSCOs_covered)]
        multiples = [curr_appear for curr_appear in number_appearences if curr_appear >= 2]
        
        ## KEEP TRACK OF WHETHER DUPLICATES/TRIPLES/ETC. ARE COMMON ##
        number_covered.append(number_appearences)
        number_duplicated.append(number_appearences.count(2))
        number_tripled.append(number_appearences.count(3))
        number_quadrupled.append(number_appearences.count(4))
        number_higher_mult.append(len([curr_appear for curr_appear in number_appearences if curr_appear > 4]))
        prop_duplicated = 0
        if len(BUSCOs_covered) > 0:
            prop_duplicated = len(multiples) / len(BUSCOs_covered) * 100
            
        percent_multiples.append(prop_duplicated)
        
        busco_completeness = len(BUSCOs_covered) / total_buscos * 100
        busco_scores.append(busco_completeness)
        levels_out.append(level_hierarchy[curr_level])
        if busco_completeness >= busco_threshold:
            success = 1
            success_level = level_hierarchy[curr_level]
        curr_level = curr_level - 1

    report_dir = os.path.join(output_dir, "busco_assessment", "output_by_level", taxonomy,
                              "_".join(organism_format.replace("(", "_").replace(")", "_").split(" ")))
    os.system("mkdir -p " + report_dir)

    report_file = os.path.join(report_dir, sample_name + "_report.txt")
    reported = open(report_file,"w")
    reversed_scores = busco_scores
    reversed_scores.reverse()
    
    if success == 1:
        file_written = os.path.join(report_dir, level_hierarchy[curr_level] + "_" + sample_name + ".txt")
        if (write_transcript_file):
            with open(file_written, 'w') as filehandle:
                for transcript_name in transcripts_to_search_sep:
                    filehandle.write(transcript_name + '\n')
        if (create_fasta):
            mock_file_name = os.path.join(report_dir, organism + "_" + level_hierarchy[curr_level] + "_" + 
                                          sample_name + "_transcripts.fasta")
            os.system("grep -w -A 2 -f " + file_written + " " + fasta_file + " --no-group-separator > " + mock_file_name)

        reported.write("Taxonomy file successfully completed with BUSCO completeness " + str(busco_completeness) + 
                       "% at location " + str(file_written) + "\n This was at taxonomic level " + str(success_level) + 
                       ". \n The file containing the transcript names for the mock transcriptome corresponding to this "
                       "taxonomic level is located here: " + str(file_written) + ".\n")
        reported.write("The BUSCO scores found at the various taxonomic levels (Supergroup to " + 
                       str(taxonomy) + ") were: " + " ".join([str(curr) for curr in reversed_scores]))
    else:
        reported.write("Sufficient BUSCO completeness not found at threshold " + str(busco_threshold) + "%. \n")
        reported.write("The BUSCO scores found at the various taxonomic levels (Supergroup to " + str(taxonomy) + 
                       ") were: " + " ".join([str(curr) for curr in reversed_scores]) + "\n")
    reported.close()
    reversed_levels = levels_out
    reversed_levels.reverse()
    return pd.DataFrame({"Organism":[organism] * len(levels_out),"TaxonomicLevel":reversed_levels,
                         "BuscoCompleteness":busco_scores,"NumberCovered":number_covered,"CtTwoCopies":number_duplicated,
                         "CtThreeCopies":number_tripled,"CtFourCopies":number_quadrupled,"CtFivePlusCopies":number_higher_mult,
                         "PercentageDuplicated":percent_multiples})

def read_in_taxonomy(infile):
    with open(infile, 'rb') as f:
        result = chardet.detect(f.read())
    tax_out = pd.read_csv(infile, sep='\t',encoding=result['encoding'])
    classes = ['supergroup','division','class','order','family','genus','species']
    for c in tax_out.columns:
        if c.lower() in classes:
            if (np.issubdtype(tax_out[str(c)].dtypes, np.number)) | (np.issubdtype(tax_out[str(c)].dtypes, np.float_)):
                tax_out = tax_out.loc[:,~(tax_out.columns == c)]
    tax_out.columns = tax_out.columns.str.lower()
    tax_out = tax_out.set_index('source_id')
    return tax_out

def queryBusco(args=None):
    """
    Search through individual BUSCO outputs to find number of matches for each organism/taxonomic level.
    """
    
    parser = argparse.ArgumentParser()
    parser.add_argument('--busco_out',default="busco", help = "The output from the BUSCO run on the full sample file.")
    parser.add_argument('--individual_or_summary','-i',default="summary",choices=["summary","individual"],
                        help = "Not necessary if we are running in summary mode.")
    parser.add_argument('--organism_group', nargs = "+", default = [],
                        help = "The focal name(s) of species/genus/order/class.")
    parser.add_argument('--taxonomic_level', nargs = "+", default = [], 
                        help = "The taxonomic level(s) of the specified focal name.")
    parser.add_argument('--fasta_file', 
                        help = "The fasta file from which we pull sequences for the mock transcriptome.")
    parser.add_argument('--taxonomy_file_prefix',
                        help = "The taxonomy file we use to create the mock transcriptome.")
    parser.add_argument('--tax_table', 
                        help = "the taxonomy table to get the full classification of the organism, " + 
                        "as assessed by the database being used.")
    parser.add_argument('--sample_name', help = "The name of the original sample being assessed.")
    parser.add_argument('--download_busco',action='store_true',
                        help = "If specified, we download BUSCO file from the url in the next argument.")
    parser.add_argument('--create_fasta',action='store_true', 
                        help = "If specified, we create a 'transcriptome fasta' when we query for the organisms.")
    parser.add_argument('--busco_url',default=0)
    parser.add_argument('--busco_location',default="busco",
                        help = "Location to store the BUSCO tar reference.")
    parser.add_argument('--output_dir',default="output")
    parser.add_argument('--available_cpus',default=1)
    parser.add_argument('--busco_threshold',default=50)
    parser.add_argument('--write_transcript_file', default=False, action='store_true',
                       help = "Whether to write an actual file with the subsetted transcriptome.")

    if args != None:
        args = parser.parse_args(args)
    else:
        args = parser.parse_args()
        
    organism = args.organism_group 
    taxonomy = args.taxonomic_level
    tax_table = read_in_taxonomy(args.tax_table)

    if (args.individual_or_summary == "individual") & ((len(args.organism_group) == 0) | (len(args.taxonomic_level) == 0)):
        print("You specified individual mode, but then did not provide a taxonomic group and/or accompanying taxonomic level.")
        sys.exit(1)
    if (len(args.organism_group) == 0) != (len(args.taxonomic_level) == 0):
        print("The number of organisms you specified is not equal to the number of taxonomic levels you specified. " +
              str(len(args.organism_group)) + " organisms were specified, while " + str(args.taxonomic_level) + 
              " taxonomic levels were specified.")
        sys.exit(1)

    if (args.individual_or_summary == "individual"):
        results_frame = Parallel(n_jobs=multiprocessing.cpu_count())(delayed(evaluate_organism)(organism[curr], 
                                                                                                taxonomy[curr], tax_table,
                                                                                                args.create_fasta, 
                                                                                                args.write_transcript_file, 
                                                                                                args.busco_out, 
                                                                                                args.taxonomy_file_prefix, 
                                                                                                float(args.busco_threshold), 
                                                                                                args.output_dir, 
                                                                                                args.sample_name, 
                                                                                                args.fasta_file) \
                                                                     for curr in range(len(organism)))
        print(results_frame,flush=True)
        results_frame = pd.concat(results_frame)
        os.system("mkdir -p " + os.path.join(args.output_dir, "busco_assessment", args.sample_name, "individual"))
        results_frame.to_csv(path_or_buf = os.path.join(args.output_dir, "busco_assessment", 
                                                        args.sample_name, "individual", "summary_" + 
                                                        args.sample_name + ".tsv"), sep = "\t")
    else:
        for taxonomy in level_hierarchy:
            taxonomy_file = pd.read_csv(args.taxonomy_file_prefix + "_all_" + str(taxonomy) + 
                                        "_counts.csv", sep=",",header=0)
            if len(taxonomy_file.index) > 0:
                curr_frame = taxonomy_file.nlargest(multiprocessing.cpu_count(), 'NumTranscripts')
                organisms = list(set(list(curr_frame[taxonomy.capitalize()])))
                results_frame = Parallel(n_jobs=multiprocessing.cpu_count())(delayed(evaluate_organism)(organism, taxonomy, tax_table, args.create_fasta, args.write_transcript_file, args.busco_out, args.taxonomy_file_prefix, args.busco_threshold, 
                                                                                                        args.output_dir, 
                                                                                                        args.sample_name, 
                                                                                                        args.fasta_file) \
                                                                             for organism in organisms)
                results_frame = pd.concat(results_frame)
            else:
                results_frame = pd.DataFrame(columns = ["Organism","TaxonomicLevel","BuscoCompleteness",
                                                        "NumberCovered","CtTwoCopies",
                                                        "CtThreeCopies","CtFourCopies","CtFivePlusCopies",
                                                        "PercentageDuplicated"])
                
            os.system("mkdir -p " + os.path.join(args.output_dir, "busco_assessment", args.sample_name, 
                                                 taxonomy + "_combined"))
            results_frame.to_csv(path_or_buf = os.path.join(args.output_dir, "busco_assessment", 
                                                            args.sample_name, taxonomy + "_combined", 
                                                            "summary_" + taxonomy + "_" +  args.sample_name + 
                                                            ".tsv"), sep = "\t")
        
    return 0
        
if __name__ == "__main__":
    queryBusco()


import pandas as pd
import os
import numpy as np
import math
import sys
import yaml
import argparse
import pathlib
from joblib import Parallel, delayed
import EUKulele

abs_path = os.path.abspath(os.path.dirname(__file__))

def eukulele(config="", string_arguments=""):
    split_args = string_arguments.split(" ")
    
    if len(split_args) > 1:
        if string_arguments.split(" ")[0] == "--config":
            stringargs = parseConfig(string_arguments.split(" ")[1])
            print("Running EUKulele with entries from the provided configuration file.")
            EUKulele.EUKulele_main.main(str(stringargs)) 
            return 0
        
    if (config == "") | (not os.path.isfile(config)):
        print("Running EUKulele with command line arguments, as no valid configuration file was provided.")
        
        EUKulele.EUKulele_main.main(str(string_arguments)) 
    else:
        print("Running EUKulele with entries from the provided configuration file.")
        args = parseConfig(config)
        EUKulele.EUKulele_main.main(str(args))   

def parseConfig(config_file):
    with open(config_file, 'r') as configfile:
        config = yaml.safe_load(configfile)

    args = ""

    ## CHECK THAT ALL OPTIONS WITHOUT DEFAULTS EXIST IN CONFIGURATION FILE ##
    required_entries = ["mets_or_mags","samples"]
    for r in required_entries:
        if r not in config:
            print("You did not include required entry: " + str(r) + " in the configuration file.")
            sys.exit(1)

    ## BASIC OPTIONS ##
    if "subroutine" in config:
        args = args + str(config["subroutine"]) + " "
    args = args + " --config_file " + str(config_file)
    args = args + " --mets_or_mags " + str(config["mets_or_mags"])
    
    ## If reference_dir is provided, databases are not downloaded.
    if "reference" in config:
        args = args + " --reference_dir " + str(config["reference"])
    if "samples" in config:
        args = args + " --sample_dir " + str(config["samples"])
    if "ref_fasta" in config: # otherwise, this will default to reference.pep.fa! Set automatically if database is auto-downloaded ("download_reference", below)
        args = args + " --ref_fasta " + str(config["ref_fasta"])
    if "output" in config:
        args = args + " --out_dir " + str(config["output"])
    if "database" in config:
        args = args + " --database " + str(config["database"])
    if "nucleotide_extension" in config:
        args = args + " --nucleotide_extension " + str(config["nucleotide_extension"])
    if "protein_extension" in config:
        args = args + " --protein_extension " + str(config["protein_extension"])
    if "scratch" in config:
        args = args + " --scratch " + str(config["scratch"])
    if "force_rerun" in config:
        if config["force_rerun"] == 1:
            args = args + " --force_rerun"

    ## SALMON OPTIONS ##
    if "use_salmon_counts" in config:
        if config["use_salmon_counts"] == 1:
            args = args + " --use_salmon_counts"
            if "salmon_dir" in config:
                args = args + " --salmon_dir " + config["salmon_dir"]
            else:
                print("You need to include a salmon directory if you wish to process salmon counts.")
                sys.exit(1)
            if "names_to_reads" in config:
                args = args + " --names_to_reads " + config["names_to_reads"]

    ## TRANSDECODER AND COMPUTATIONS OPTIONS ##
    if "transdecoder_orfsize" in config:
        args = args + " --transdecoder_orfsize " + str(config["transdecoder_orfsize"])
    if "CPUs" in config:
        args = args + " --CPUs " + str(config["CPUs"])
    if "run_transdecoder" in config:
        args = args + " --run_transdecoder"

    ## ALIGNMENT AND BUSCO OPTIONS ##
    if "alignment_choice" in config: 
        args = args + " --alignment_choice " + str(config["alignment_choice"])
    if "cutoff" in config:    
        args = args + " --cutoff_file " + config["cutoff"]
    if "filter_metric" in config:
        args = args + " --filter_metric " + config["filter_metric"]
    if "consensus_cutoff" in config:
        args = args + " --consensus_cutoff " + str(config["consensus_cutoff"])
    if "busco_file" in config:
        args = args + " --busco_file " + str(config["busco_file"])
    if "individual_or_summary" in config:
        args = args + " --individual_or_summary " + str(config["individual_or_summary"])
    if "organisms" in config:
        args = args + " --organisms " + str(" ".join(config["organisms"]))
    if "taxonomy_organisms" in config:
        args = args + " --taxonomy_organisms " + str(" ".join(config["taxonomy_organisms"]))
    if "busco_threshold" in config:
        args = args + " --busco_threshold " + str(config["busco_threshold"])

    if ("tax_table" in config): # unique, non-default name for formatted taxonomy table
        tax_table = config["tax_table"]
        args = args + " --tax_table " + str(tax_table)
    if ("protein_map" in config): # unique, non-default name for formatted protein map
        protein_map = config["protein_map"]
        args = args + " --protein_map " + str(protein_map)
        
    return args
     
if __name__ == "__main__": 
    eukulele(string_arguments = " ".join(sys.argv))


import os
import sys
import yaml
import subprocess
import pkgutil

import EUKulele
from EUKulele.manage_steps import createAlignmentDatabase

from scripts.create_protein_table import createProteinTable

def downloadDatabase(database_name, alignment_choice):
    """
    Automatically downloads a peptide database for use with EUKulele and stores the name of
    the resulting FASTA file and taxonomy table.
    """
    
    print("Automatically downloading database " + database_name + ". If you intended to " + \
          "use an existing database folder, be sure a reference FASTA, protein map, and taxonomy table " + \
          "are provided. Check the documentation for details.")
    create_protein_table_args = []
    if database_name == "mmetsp":
        column_id = "SOURCE_ID"
        delimiter = "/"
    elif (database_name == "eukprot"):
        column_id = 0
        delimiter = "\t"
        create_protein_table_args.append("--euk-prot")
    elif (database_name == "phylodb"):
        column_id = 0
        delimiter = "\t"
        create_protein_table_args.append("--reformat_tax")
    else:
        print("Specified reference database, " + database_name + " is not supported.")
        sys.exit(1)
        
    #config_file = pkgutil.get_data(__name__, "static/reference_url.yaml")
    config_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "static/reference_url.yaml")
    with open(config_file, 'r') as configfile:
        config = yaml.safe_load(configfile)
        
    database_ref_url = config[database_name + "_ref"]
    database_tab_url = config[database_name + "_tab"]
    rc = os.system(" ".join(["download_database.sh", database_name, database_ref_url, 
                          database_tab_url]))
    if rc != 0:
        print("Download of database " + database_name + " did not complete correctly.")
        sys.exit(1)
        
    fasta_name = "reference.pep.fa" #os.path.join(database_name,"reference.pep.fa")
    orig_tax_name = os.path.join(database_name,"taxonomy-table.txt")
    
    tax_table = os.path.join(database_name,"tax-table.txt")
    protein_json = os.path.join(database_name,"prot-map.json")
    
    create_protein_table_args.extend(["--infile_peptide",os.path.join(database_name,fasta_name),
                                      "--infile_taxonomy",orig_tax_name,"--output",tax_table,"--outfile_json",
                                      protein_json,"--delim",str(delimiter),"--strain_col_id",
                                      "strain_name","--taxonomy_col_id", "taxonomy","--column",
                                      str(column_id)])
    
    ## Run function to create protein table file from scripts/create_protein_table.py ##
    rc1 = createProteinTable(create_protein_table_args)
    if rc1 != 0:
        print("Taxonomy table and protein JSON file creation step did not complete successfully.")
        sys.exit(1)
        
    rc2 = createAlignmentDatabase(fasta_name.split("/")[-1], True, alignment_choice, database_name)
    if rc2 != 0:
        print("Alignment database for " + alignment_choice + " did not complete successfully; " +
              "check log for details.")
        sys.exit(1)
    
    return fasta_name, tax_table, protein_json


import os
import sys
import subprocess
import multiprocessing
from joblib import Parallel, delayed
import pandas as pd
import math

MEM_AVAIL_GB = 0
while MEM_AVAIL_GB == 0:
    try:
        os.system("free -m > free.csv")
        MEM_AVAIL_GB = pd.read_csv("free.csv", sep = "\s+").free[0] / 10**3
    except:
        pass
MAX_JOBS = math.floor(MEM_AVAIL_GB / 80)
if MAX_JOBS == 0:
    MAX_JOBS = 1

from scripts.query_busco import queryBusco

def readBuscoFile(individual_or_summary, busco_file, organisms, organisms_taxonomy):
    if individual_or_summary == "individual":
        if (busco_file != "") & (os.path.isfile(busco_file)):
            busco_file_read = pd.read_csv(busco_file, sep = "\t")
            organisms = list(busco_file_read.iloc[:,0])
            organisms_taxonomy = list(busco_file_read.iloc[:,1])
            print("Organisms and their taxonomy levels for BUSCO analysis were read from file.")
            
            if (len(organisms) != len(organisms_taxonomy)):
                print("Organisms and taxonomic specifications for BUSCO analysis do not contain the same number of entries. " + \
                      "Please revise such that each organism flagged for BUSCO analysis also includes its original " + \
                      "taxonomic level.")
                sys.exit(1)
        else:
            print("No BUSCO file specified/found; using argument-specified organisms and taxonomy for BUSCO analysis.")
            
    return organisms, organisms_taxonomy


def configRunBusco(output_dir, mets_or_mags, pep_ext, nt_ext, sample_dir, samples):
    print("Performing BUSCO steps...", flush=True)
    print("Configuring BUSCO...", flush=True)
    
    ## Run BUSCO on the full dataset ##
    busco_db = "eukaryota_odb10"
    busco_config_res = configure_busco(busco_db)
    n_jobs_busco = min(multiprocessing.cpu_count(), len(samples), MAX_JOBS)
    print("Running busco...", flush=True)
    busco_res = Parallel(n_jobs=n_jobs_busco, prefer="threads")(delayed(run_busco)(sample_name, 
                                                                                                  os.path.join(output_dir, 
                                                                                                               "busco"), 
                                                                                                  output_dir,
                                                                                                  busco_db, mets_or_mags, 
                                                                                                  pep_ext, nt_ext,
                                                                                                  sample_dir) \
                                                                               for sample_name in samples)
    all_codes = sum(busco_res) + busco_config_res
    if sum(busco_res) > 0:
        print("BUSCO initial run did not complete successfully.\n" + 
              "Please check the BUSCO run log files in the log/ folder.", flush = True)
        sys.exit(1)
    if busco_config_res > 0:
        print("BUSCO initial configuration did not complete successfully.\n" + 
              "Please check the BUSCO configuration log files in the log/ folder.", flush = True)
        sys.exit(1)
                
def configure_busco(busco_db):
    busco_config_log = open(os.path.join("log","busco_config.out"), "w+")
    busco_config_err = open(os.path.join("log","busco_config.err"), "w+")
    rc1 = 0
    
    if not os.path.isdir(os.path.join("busco_downloads","lineages","eukaryota_odb10")):
        p1 = subprocess.Popen(["configure_busco.sh", busco_db], stdout = busco_config_log, stderr = busco_config_err)
        p1.wait()
        rc1 = p1.returncode
    else:
        print("BUSCO lineage database already found; not re-downloaded.")
        
    busco_config_log.close()
    busco_config_err.close()
    return rc1
        
def run_busco(sample_name, output_dir_busco, output_dir, busco_db, mets_or_mags, pep_ext, nt_ext, sample_dir):
    CPUS = multiprocessing.cpu_count()
    
    if mets_or_mags == "mets":
        if os.path.isfile(os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext)):
            fastaname = os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext) 
        else:
            fastaname = os.path.join(sample_dir, sample_name + "." + nt_ext)
    else:
        fastaname = os.path.join(sample_dir, sample_name + "." + pep_ext)
        
    busco_run_log = open(os.path.join("log","busco_run.out"), "w+")
    busco_run_err = open(os.path.join("log","busco_run.err"), "w+")
    p1 = subprocess.Popen(["run_busco.sh", str(sample_name), str(output_dir_busco), 
                              os.path.join(output_dir_busco, "config_" + sample_name + ".ini"), 
                              fastaname, str(CPUS), busco_db], stdout = busco_run_log, stderr = busco_run_err)
    
    ## TRAVIS DEBUGGING!! ##
    
    p1.wait()
    rc1 = p1.returncode
    
    busco_run_log.close()
    busco_run_err.close()
    
    
    a_file = open(os.path.join("log","busco_run.err"))

    lines = a_file.readlines()
    for line in lines:
        print(line)
        
    return rc1 

def manageBuscoQuery(output_dir, individual_or_summary, samples, mets_or_mags, pep_ext, nt_ext,
                     sample_dir, organisms, organisms_taxonomy, tax_tab, busco_threshold):
    """
    Assess BUSCO completeness on the most prevalent members of the metatranscriptome at each taxonomic level.
    """
    
    if individual_or_summary == "individual":
        if (len(organisms) != len(organisms_taxonomy)):
            print("A different number of organisms was specified than the taxonomic levels given in " + 
                  "individual mode. Please check inputs.")
            sys.exit(1)
        if (len(organisms) == 0) | (len(organisms_taxonomy) == 0):
            print("The number of organisms specified was " + str(len(organisms)) +
                  " and the number of taxonomic levels specified was " + str(len(organisms_taxonomy)) + 
                  " in individual mode. Neither can be zero. Please check inputs.")
            sys.exit(1)
            
        for sample_name in samples:
            # the BUSCO table that we're interested in using that contains the BUSCO matches and their level of completeness
            busco_table = os.path.join(output_dir, "busco", sample_name, "full_table.tsv") 
            # the prefix to specify where the taxonomy estimation output files are located
            taxfile_stub = os.path.join(output_dir, "taxonomy_counts", output_dir.split("/")[-1]) 

            if mets_or_mags == "mets":
                if os.path.isfile(os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext)):
                    fasta = os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext) 
                else:
                    fasta = os.path.join(sample_dir, sample_name + "." + nt_ext)
            else:
                fasta = os.path.join(sample_dir, sample_name + "." + pep_ext)

            query_busco_log = open(os.path.join("log","busco_query_" + sample_name + ".log"), "w+")
            query_busco_err = open(os.path.join("log","busco_query_" + sample_name + ".err"), "w+")
            sys.stdout = query_busco_log
            sys.stderr = query_busco_err
            query_args = ["--organism_group",str(" ".join(organisms)),"--taxonomic_level",
                          str(" ".join(organisms_taxonomy)),"--output_dir",output_dir,"--fasta_file",
                          fasta,"--sample_name",sample_name,"--taxonomy_file_prefix",taxfile_stub,
                          "--tax_table",tax_tab,"--busco_out",busco_table,"-i","individual",
                          "--busco_threshold",str(busco_threshold)]
            try:
                rc = queryBusco(query_args)
            except:
                print("BUSCO query did not run successfully for sample " + sample_name + "; check log file for details.")
                sys.exit(1)

            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__ 
            #if (rc != 0) | (os.stat(os.path.join("log","busco_query_" + sample_name + ".err")).st_size != 0):
            #    print("BUSCO query did not run successfully for sample " + sample_name + "; check log file for details.")
            #    sys.exit(1)
    else:
        for sample_name in samples:
            # the BUSCO table that we're interested in using that contains the BUSCO matches and their level of completeness
            busco_table = os.path.join(output_dir, "busco", sample_name, "full_table.tsv") 
            # the prefix to specify where the taxonomy estimation output files are located
            taxfile_stub = os.path.join(output_dir, "taxonomy_counts", output_dir.split("/")[-1])

            if mets_or_mags == "mets":
                fasta = os.path.join(output_dir, mets_or_mags, sample_name + "." + pep_ext) 
            else:
                fasta = os.path.join(sample_dir, sample_name + "." + nt_ext)

            query_busco_log = open(os.path.join("log","busco_query_" + sample_name + ".log"), "w+")
            query_busco_err = open(os.path.join("log","busco_query_" + sample_name + ".err"), "w+")
            sys.stdout = query_busco_log
            sys.stderr = query_busco_err
            query_args = ["--output_dir",output_dir,"--fasta_file",fasta,"--sample_name",
                          sample_name,"--taxonomy_file_prefix",taxfile_stub,"--tax_table",
                          tax_tab,"--busco_out",busco_table,"-i","summary"]

            try:
                rc = queryBusco(query_args)
            except OSError as e:
                print("Not all files needed to run BUSCO query (output of BUSCO run) found; check log file for details.")
                rc = 1
            except:
                print("Unexpected error:", sys.exc_info()[0])
                rc = 1
                
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__ 

            if (rc != 0) | (os.stat(os.path.join("log","busco_query_" + sample_name + ".err")).st_size != 0):
                print("BUSCO query did not run successfully for sample " + sample_name + "; check log file for details.")
                sys.exit(1)
            else:
                print("BUSCO query complete.")


from setuptools import setup, find_packages
import os
import numpy.random
from glob import glob

os.system("export PY_IGNORE_IMPORTMISMATCH=1")
version = open('VERSION').read().strip()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name="EUKulele",
    version=version,
    description = ("A package to make the process of taxonomically classifying "
                   "microbial eukaryotes easier."),
    keywords = "eukaryote taxonomy classification",
    conda_buildnum=1,
    url="https://github.com/AlexanderLabWHOI/EUKulele",
    author="Arianna Krinos",
    author_email="akrinos@mit.edu",
    packages=['EUKulele','scripts'],
    package_dir={'EUKulele': 'src/EUKulele'},
    scripts=['bin/EUKulele','scripts/create_protein_table.py','scripts/download_database.sh',\
             'scripts/concatenate_busco.sh','scripts/configure_busco.sh','scripts/run_busco.sh',\
             'scripts/install_dependencies.sh','scripts/after_job.sh','scripts/coordinate_batch.sh'],
    license="MIT",
    include=['code'],
    include_package_data = True,
    setup_requires=['pytest-runner'],
    test_suite='tests',
    tests_require=['pytest'],
    install_requires=required,
    zip_safe=False,
)
